@@ -10,14 +10,18 @@ needs to be updated.
 When adding a function, make sure to use the next integer not used as an index
 (in case you use an existing index or jump, the build will stop and raise an
 exception, so it should hopefully not get unnoticed).
+
 """
+from __future__ import division, absolute_import, print_function
 
 multiarray_global_vars = {
     'NPY_NUMUSERTYPES':             7,
+    'NPY_DEFAULT_ASSIGN_CASTING':   292,
 }
 
 multiarray_global_vars_types = {
     'NPY_NUMUSERTYPES':             'int',
+    'NPY_DEFAULT_ASSIGN_CASTING':   'NPY_CASTING',
 }
 
 multiarray_scalar_bool_values = {
@@ -68,6 +72,7 @@ multiarray_types_api = {
     'PyTimedeltaArrType_Type':          216,
     'PyHalfArrType_Type':               217,
     'NpyIter_Type':                     218,
+    # End 1.6 API
 }
 
 #define NPY_NUMUSERTYPES (*(int *)PyArray_API[6])
@@ -256,7 +261,7 @@ multiarray_funcs_api = {
     'PyArray_TimedeltaToTimedeltaStruct':   221,
     'PyArray_DatetimeStructToDatetime':     222,
     'PyArray_TimedeltaStructToTimedelta':   223,
-    # New Iterator API
+    # NDIter API
     'NpyIter_New':                          224,
     'NpyIter_MultiNew':                     225,
     'NpyIter_AdvancedNew':                  226,
@@ -315,6 +320,27 @@ multiarray_funcs_api = {
     'PyArray_GetArrayParamsFromObject':     278,
     'PyArray_ConvertClipmodeSequence':      279,
     'PyArray_MatrixProduct2':               280,
+    # End 1.6 API
+    'NpyIter_IsFirstVisit':                 281,
+    'PyArray_SetBaseObject':                282,
+    'PyArray_CreateSortedStridePerm':       283,
+    'PyArray_RemoveAxesInPlace':            284,
+    'PyArray_DebugPrint':                   285,
+    'PyArray_FailUnlessWriteable':          286,
+    'PyArray_SetUpdateIfCopyBase':          287,
+    'PyDataMem_NEW':                        288,
+    'PyDataMem_FREE':                       289,
+    'PyDataMem_RENEW':                      290,
+    'PyDataMem_SetEventHook':               291,
+    'PyArray_MapIterSwapAxes':              293,
+    'PyArray_MapIterArray':                 294,
+    'PyArray_MapIterNext':                  295,
+    # End 1.7 API
+    'PyArray_Partition':                    296,
+    'PyArray_ArgPartition':                 297,
+    'PyArray_SelectkindConverter':          298,
+    'PyDataMem_NEW_ZEROED':                 299,
+    # End 1.8 API
 }
 
 ufunc_types_api = {
@@ -361,6 +387,12 @@ ufunc_funcs_api = {
     'PyUFunc_ee_e':                             36,
     'PyUFunc_ee_e_As_ff_f':                     37,
     'PyUFunc_ee_e_As_dd_d':                     38,
+    # End 1.6 API
+    'PyUFunc_DefaultTypeResolver':              39,
+    'PyUFunc_ValidateCasting':                  40,
+    # End 1.7 API
+    'PyUFunc_RegisterLoopForDescr':             41,
+    # End 1.8 API
 }
 
 # List of all the dicts which define the C API
