@@ -7,35 +7,26 @@ extern NPY_NO_EXPORT PyMappingMethods array_as_mapping;
 NPY_NO_EXPORT PyMappingMethods array_as_mapping;
 #endif
 
-NPY_NO_EXPORT PyObject *
-array_big_item(PyArrayObject *self, intp i);
-
 NPY_NO_EXPORT Py_ssize_t
 array_length(PyArrayObject *self);
 
 NPY_NO_EXPORT PyObject *
-array_item_nice(PyArrayObject *self, Py_ssize_t i);
+array_item_asarray(PyArrayObject *self, npy_intp i);
+
+NPY_NO_EXPORT PyObject *
+array_item_asscalar(PyArrayObject *self, npy_intp i);
+
+NPY_NO_EXPORT PyObject *
+array_item(PyArrayObject *self, Py_ssize_t i);
+
+NPY_NO_EXPORT PyObject *
+array_subscript_asarray(PyArrayObject *self, PyObject *op);
 
 NPY_NO_EXPORT PyObject *
 array_subscript(PyArrayObject *self, PyObject *op);
 
 NPY_NO_EXPORT int
-array_ass_big_item(PyArrayObject *self, intp i, PyObject *v);
-
-#if PY_VERSION_HEX < 0x02050000
-        #if SIZEOF_INT == SIZEOF_INTP
-                #define array_ass_item array_ass_big_item
-        #endif
-#else
-        #if SIZEOF_SIZE_T == SIZEOF_INTP
-                #define array_ass_item array_ass_big_item
-        #endif
-#endif
-#ifndef array_ass_item
-NPY_NO_EXPORT int
-_array_ass_item(PyArrayObject *self, Py_ssize_t i, PyObject *v);
-#define array_ass_item _array_ass_item
-#endif
+array_ass_item(PyArrayObject *self, Py_ssize_t i, PyObject *v);
 
 NPY_NO_EXPORT PyObject *
 add_new_axes_0d(PyArrayObject *,  int);
@@ -53,7 +44,7 @@ PyArray_MapIterReset(PyArrayMapIterObject *mit);
 NPY_NO_EXPORT void
 PyArray_MapIterNext(PyArrayMapIterObject *mit);
 
-NPY_NO_EXPORT void
+NPY_NO_EXPORT int
 PyArray_MapIterBind(PyArrayMapIterObject *, PyArrayObject *);
 
 NPY_NO_EXPORT PyObject*
