@@ -3,12 +3,14 @@
 #include "structmember.h"
 
 /*#include <stdio.h>*/
+#define NPY_NO_DEPRECATED_API NPY_API_VERSION
 #define _MULTIARRAYMODULE
-#define NPY_NO_PREFIX
 #include "numpy/arrayobject.h"
 
 #include "npy_config.h"
 
+#include "npy_pycompat.h"
+
 #include "number.h"
 
 /*************************************************************************
@@ -24,12 +26,13 @@ NPY_NO_EXPORT NumericOps n_ops; /* NB: static objects initialized to zero */
 
 /* FIXME - macro contains a return */
 #define SET(op)   temp = PyDict_GetItemString(dict, #op); \
-    if (temp != NULL) {                                   \
-        if (!(PyCallable_Check(temp))) {                  \
-            return -1;                                    \
-        }                                                 \
-        Py_XDECREF(n_ops.op);                             \
-        n_ops.op = temp;                                  \
+    if (temp != NULL) { \
+        if (!(PyCallable_Check(temp))) { \
+            return -1; \
+        } \
+        Py_INCREF(temp); \
+        Py_XDECREF(n_ops.op); \
+        n_ops.op = temp; \
     }
 
 
@@ -48,7 +51,7 @@ PyArray_SetNumericOps(PyObject *dict)
     SET(power);
     SET(square);
     SET(reciprocal);
-    SET(ones_like);
+    SET(_ones_like);
     SET(sqrt);
     SET(negative);
     SET(absolute);
@@ -99,7 +102,7 @@ PyArray_GetNumericOps(void)
     GET(power);
     GET(square);
     GET(reciprocal);
-    GET(ones_like);
+    GET(_ones_like);
     GET(sqrt);
     GET(negative);
     GET(absolute);
@@ -136,9 +139,9 @@ static PyObject *
 _get_keywords(int rtype, PyArrayObject *out)
 {
     PyObject *kwds = NULL;
-    if (rtype != PyArray_NOTYPE || out != NULL) {
+    if (rtype != NPY_NOTYPE || out != NULL) {
         kwds = PyDict_New();
-        if (rtype != PyArray_NOTYPE) {
+        if (rtype != NPY_NOTYPE) {
             PyArray_Descr *descr;
             descr = PyArray_DescrFromType(rtype);
             if (descr) {
@@ -206,6 +209,26 @@ PyArray_GenericBinaryFunction(PyArrayObject *m1, PyObject *m2, PyObject *op)
         Py_INCREF(Py_NotImplemented);
         return Py_NotImplemented;
     }
+
+    if (!PyArray_Check(m2)) {
+          /*
+           * Catch priority inversion and punt, but only if it's guaranteed
+           * that we were called through m1 and the other guy is not an array
+           * at all. Note that some arrays need to pass through here even
+           * with priorities inverted, for example: float(17) * np.matrix(...)
+           *
+           * See also:
+           * - https://github.com/numpy/numpy/issues/3502
+           * - https://github.com/numpy/numpy/issues/3503
+           */
+          double m1_prio = PyArray_GetPriority(m1, NPY_SCALAR_PRIORITY);
+          double m2_prio = PyArray_GetPriority(m2, NPY_SCALAR_PRIORITY);
+          if (m1_prio < m2_prio) {
+              Py_INCREF(Py_NotImplemented);
+              return Py_NotImplemented;
+          }
+    }
+
     return PyObject_CallFunction(op, "OO", m1, m2);
 }
 
@@ -258,11 +281,13 @@ array_multiply(PyArrayObject *m1, PyObject *m2)
     return PyArray_GenericBinaryFunction(m1, m2, n_ops.multiply);
 }
 
+#if !defined(NPY_PY3K)
 static PyObject *
 array_divide(PyArrayObject *m1, PyObject *m2)
 {
     return PyArray_GenericBinaryFunction(m1, m2, n_ops.divide);
 }
+#endif
 
 static PyObject *
 array_remainder(PyArrayObject *m1, PyObject *m2)
@@ -270,64 +295,91 @@ array_remainder(PyArrayObject *m1, PyObject *m2)
     return PyArray_GenericBinaryFunction(m1, m2, n_ops.remainder);
 }
 
-static int
-array_power_is_scalar(PyObject *o2, double* exp)
+/* Determine if object is a scalar and if so, convert the object
+ *   to a double and place it in the out_exponent argument
+ *   and return the "scalar kind" as a result.   If the object is
+ *   not a scalar (or if there are other error conditions)
+ *   return NPY_NOSCALAR, and out_exponent is undefined.
+ */
+static NPY_SCALARKIND
+is_scalar_with_conversion(PyObject *o2, double* out_exponent)
 {
     PyObject *temp;
     const int optimize_fpexps = 1;
 
     if (PyInt_Check(o2)) {
-        *exp = (double)PyInt_AsLong(o2);
-        return 1;
+        *out_exponent = (double)PyInt_AsLong(o2);
+        return NPY_INTPOS_SCALAR;
     }
     if (optimize_fpexps && PyFloat_Check(o2)) {
-        *exp = PyFloat_AsDouble(o2);
-        return 1;
+        *out_exponent = PyFloat_AsDouble(o2);
+        return NPY_FLOAT_SCALAR;
     }
-    if ((PyArray_IsZeroDim(o2) &&
-         ((PyArray_ISINTEGER(o2) ||
-           (optimize_fpexps && PyArray_ISFLOAT(o2))))) ||
-        PyArray_IsScalar(o2, Integer) ||
-        (optimize_fpexps && PyArray_IsScalar(o2, Floating))) {
-        temp = o2->ob_type->tp_as_number->nb_float(o2);
-        if (temp != NULL) {
-            *exp = PyFloat_AsDouble(o2);
+    if (PyArray_Check(o2)) {
+        if ((PyArray_NDIM(o2) == 0) &&
+                ((PyArray_ISINTEGER((PyArrayObject *)o2) ||
+                 (optimize_fpexps && PyArray_ISFLOAT((PyArrayObject *)o2))))) {
+            temp = Py_TYPE(o2)->tp_as_number->nb_float(o2);
+            if (temp == NULL) {
+                return NPY_NOSCALAR;
+            }
+            *out_exponent = PyFloat_AsDouble(o2);
             Py_DECREF(temp);
-            return 1;
+            if (PyArray_ISINTEGER((PyArrayObject *)o2)) {
+                return NPY_INTPOS_SCALAR;
+            }
+            else { /* ISFLOAT */
+                return NPY_FLOAT_SCALAR;
+            }
         }
     }
-#if (PY_VERSION_HEX >= 0x02050000)
-    if (PyIndex_Check(o2)) {
+    else if (PyArray_IsScalar(o2, Integer) ||
+                (optimize_fpexps && PyArray_IsScalar(o2, Floating))) {
+        temp = Py_TYPE(o2)->tp_as_number->nb_float(o2);
+        if (temp == NULL) {
+            return NPY_NOSCALAR;
+        }
+        *out_exponent = PyFloat_AsDouble(o2);
+        Py_DECREF(temp);
+
+        if (PyArray_IsScalar(o2, Integer)) {
+                return NPY_INTPOS_SCALAR;
+        }
+        else { /* IsScalar(o2, Floating) */
+            return NPY_FLOAT_SCALAR;
+        }
+    }
+    else if (PyIndex_Check(o2)) {
         PyObject* value = PyNumber_Index(o2);
         Py_ssize_t val;
         if (value==NULL) {
             if (PyErr_Occurred()) {
                 PyErr_Clear();
             }
-            return 0;
+            return NPY_NOSCALAR;
         }
         val = PyInt_AsSsize_t(value);
         if (val == -1 && PyErr_Occurred()) {
             PyErr_Clear();
-            return 0;
+            return NPY_NOSCALAR;
         }
-        *exp = (double) val;
-        return 1;
+        *out_exponent = (double) val;
+        return NPY_INTPOS_SCALAR;
     }
-#endif
-    return 0;
+    return NPY_NOSCALAR;
 }
 
 /* optimize float array or complex array to a scalar power */
 static PyObject *
 fast_scalar_power(PyArrayObject *a1, PyObject *o2, int inplace)
 {
-    double exp;
+    double exponent;
+    NPY_SCALARKIND kind;   /* NPY_NOSCALAR is not scalar */
 
-    if (PyArray_Check(a1) && array_power_is_scalar(o2, &exp)) {
+    if (PyArray_Check(a1) && ((kind=is_scalar_with_conversion(o2, &exponent))>0)) {
         PyObject *fastop = NULL;
         if (PyArray_ISFLOAT(a1) || PyArray_ISCOMPLEX(a1)) {
-            if (exp == 1.0) {
+            if (exponent == 1.0) {
                 /* we have to do this one special, as the
                    "copy" method of array objects isn't set
                    up early enough to be added
@@ -340,16 +392,16 @@ fast_scalar_power(PyArrayObject *a1, PyObject *o2, int inplace)
                     return PyArray_Copy(a1);
                 }
             }
-            else if (exp == -1.0) {
+            else if (exponent == -1.0) {
                 fastop = n_ops.reciprocal;
             }
-            else if (exp ==  0.0) {
-                fastop = n_ops.ones_like;
+            else if (exponent ==  0.0) {
+                fastop = n_ops._ones_like;
             }
-            else if (exp ==  0.5) {
+            else if (exponent ==  0.5) {
                 fastop = n_ops.sqrt;
             }
-            else if (exp ==  2.0) {
+            else if (exponent ==  2.0) {
                 fastop = n_ops.square;
             }
             else {
@@ -362,15 +414,36 @@ fast_scalar_power(PyArrayObject *a1, PyObject *o2, int inplace)
                 return PyArray_GenericUnaryFunction(a1, fastop);
             }
         }
-        else if (exp==2.0) {
+        /* Because this is called with all arrays, we need to
+         *  change the output if the kind of the scalar is different
+         *  than that of the input and inplace is not on ---
+         *  (thus, the input should be up-cast)
+         */
+        else if (exponent == 2.0) {
             fastop = n_ops.multiply;
             if (inplace) {
                 return PyArray_GenericInplaceBinaryFunction
                     (a1, (PyObject *)a1, fastop);
             }
             else {
-                return PyArray_GenericBinaryFunction
-                    (a1, (PyObject *)a1, fastop);
+                PyArray_Descr *dtype = NULL;
+                PyObject *res;
+
+                /* We only special-case the FLOAT_SCALAR and integer types */
+                if (kind == NPY_FLOAT_SCALAR && PyArray_ISINTEGER(a1)) {
+                    dtype = PyArray_DescrFromType(NPY_DOUBLE);
+                    a1 = (PyArrayObject *)PyArray_CastToType(a1, dtype,
+                            PyArray_ISFORTRAN(a1));
+                    if (a1 == NULL) {
+                        return NULL;
+                    }
+                }
+                else {
+                    Py_INCREF(a1);
+                }
+                res = PyArray_GenericBinaryFunction(a1, (PyObject *)a1, fastop);
+                Py_DECREF(a1);
+                return res;
             }
         }
     }
@@ -456,11 +529,13 @@ array_inplace_multiply(PyArrayObject *m1, PyObject *m2)
     return PyArray_GenericInplaceBinaryFunction(m1, m2, n_ops.multiply);
 }
 
+#if !defined(NPY_PY3K)
 static PyObject *
 array_inplace_divide(PyArrayObject *m1, PyObject *m2)
 {
     return PyArray_GenericInplaceBinaryFunction(m1, m2, n_ops.divide);
 }
+#endif
 
 static PyObject *
 array_inplace_remainder(PyArrayObject *m1, PyObject *m2)
@@ -539,11 +614,11 @@ array_inplace_true_divide(PyArrayObject *m1, PyObject *m2)
 static int
 _array_nonzero(PyArrayObject *mp)
 {
-    intp n;
+    npy_intp n;
 
     n = PyArray_SIZE(mp);
     if (n == 1) {
-        return mp->descr->f->nonzero(mp->data, mp);
+        return PyArray_DESCR(mp)->f->nonzero(PyArray_DATA(mp), mp);
     }
     else if (n == 0) {
         return 0;
@@ -568,11 +643,18 @@ array_divmod(PyArrayObject *op1, PyObject *op2)
     if (divp == NULL) {
         return NULL;
     }
+    else if(divp == Py_NotImplemented) {
+        return divp;
+    }
     modp = array_remainder(op1, op2);
     if (modp == NULL) {
         Py_DECREF(divp);
         return NULL;
     }
+    else if(modp == Py_NotImplemented) {
+        Py_DECREF(divp);
+        return modp;
+    }
     result = Py_BuildValue("OO", divp, modp);
     Py_DECREF(divp);
     Py_DECREF(modp);
@@ -589,24 +671,34 @@ array_int(PyArrayObject *v)
                         " converted to Python scalars");
         return NULL;
     }
-    pv = v->descr->f->getitem(v->data, v);
+    pv = PyArray_DESCR(v)->f->getitem(PyArray_DATA(v), v);
     if (pv == NULL) {
         return NULL;
     }
-    if (pv->ob_type->tp_as_number == 0) {
+    if (Py_TYPE(pv)->tp_as_number == 0) {
         PyErr_SetString(PyExc_TypeError, "cannot convert to an int; "\
                         "scalar object is not a number");
         Py_DECREF(pv);
         return NULL;
     }
-    if (pv->ob_type->tp_as_number->nb_int == 0) {
+    if (Py_TYPE(pv)->tp_as_number->nb_int == 0) {
         PyErr_SetString(PyExc_TypeError, "don't know how to convert "\
                         "scalar number to int");
         Py_DECREF(pv);
         return NULL;
     }
+    /*
+     * If we still got an array which can hold references, stop
+     * because it could point back at 'v'.
+     */
+    if (PyArray_Check(pv) &&
+                PyDataType_REFCHK(PyArray_DESCR((PyArrayObject *)pv))) {
+        PyErr_SetString(PyExc_TypeError,
+                "object array may be self-referencing");
+        return NULL;
+    }
 
-    pv2 = pv->ob_type->tp_as_number->nb_int(pv);
+    pv2 = Py_TYPE(pv)->tp_as_number->nb_int(pv);
     Py_DECREF(pv);
     return pv2;
 }
@@ -620,27 +712,39 @@ array_float(PyArrayObject *v)
                         "be converted to Python scalars");
         return NULL;
     }
-    pv = v->descr->f->getitem(v->data, v);
+    pv = PyArray_DESCR(v)->f->getitem(PyArray_DATA(v), v);
     if (pv == NULL) {
         return NULL;
     }
-    if (pv->ob_type->tp_as_number == 0) {
+    if (Py_TYPE(pv)->tp_as_number == 0) {
         PyErr_SetString(PyExc_TypeError, "cannot convert to a "\
                         "float; scalar object is not a number");
         Py_DECREF(pv);
         return NULL;
     }
-    if (pv->ob_type->tp_as_number->nb_float == 0) {
+    if (Py_TYPE(pv)->tp_as_number->nb_float == 0) {
         PyErr_SetString(PyExc_TypeError, "don't know how to convert "\
                         "scalar number to float");
         Py_DECREF(pv);
         return NULL;
     }
-    pv2 = pv->ob_type->tp_as_number->nb_float(pv);
+    /*
+     * If we still got an array which can hold references, stop
+     * because it could point back at 'v'.
+     */
+    if (PyArray_Check(pv) &&
+                    PyDataType_REFCHK(PyArray_DESCR((PyArrayObject *)pv))) {
+        PyErr_SetString(PyExc_TypeError,
+                "object array may be self-referencing");
+        return NULL;
+    }
+    pv2 = Py_TYPE(pv)->tp_as_number->nb_float(pv);
     Py_DECREF(pv);
     return pv2;
 }
 
+#if !defined(NPY_PY3K)
+
 static PyObject *
 array_long(PyArrayObject *v)
 {
@@ -650,18 +754,28 @@ array_long(PyArrayObject *v)
                         "be converted to Python scalars");
         return NULL;
     }
-    pv = v->descr->f->getitem(v->data, v);
-    if (pv->ob_type->tp_as_number == 0) {
+    pv = PyArray_DESCR(v)->f->getitem(PyArray_DATA(v), v);
+    if (Py_TYPE(pv)->tp_as_number == 0) {
         PyErr_SetString(PyExc_TypeError, "cannot convert to an int; "\
                         "scalar object is not a number");
         return NULL;
     }
-    if (pv->ob_type->tp_as_number->nb_long == 0) {
+    if (Py_TYPE(pv)->tp_as_number->nb_long == 0) {
         PyErr_SetString(PyExc_TypeError, "don't know how to convert "\
                         "scalar number to long");
         return NULL;
     }
-    pv2 = pv->ob_type->tp_as_number->nb_long(pv);
+    /*
+     * If we still got an array which can hold references, stop
+     * because it could point back at 'v'.
+     */
+    if (PyArray_Check(pv) &&
+                    PyDataType_REFCHK(PyArray_DESCR((PyArrayObject *)pv))) {
+        PyErr_SetString(PyExc_TypeError,
+                "object array may be self-referencing");
+        return NULL;
+    }
+    pv2 = Py_TYPE(pv)->tp_as_number->nb_long(pv);
     Py_DECREF(pv);
     return pv2;
 }
@@ -675,18 +789,28 @@ array_oct(PyArrayObject *v)
                         "be converted to Python scalars");
         return NULL;
     }
-    pv = v->descr->f->getitem(v->data, v);
-    if (pv->ob_type->tp_as_number == 0) {
+    pv = PyArray_DESCR(v)->f->getitem(PyArray_DATA(v), v);
+    if (Py_TYPE(pv)->tp_as_number == 0) {
         PyErr_SetString(PyExc_TypeError, "cannot convert to an int; "\
                         "scalar object is not a number");
         return NULL;
     }
-    if (pv->ob_type->tp_as_number->nb_oct == 0) {
+    if (Py_TYPE(pv)->tp_as_number->nb_oct == 0) {
         PyErr_SetString(PyExc_TypeError, "don't know how to convert "\
                         "scalar number to oct");
         return NULL;
     }
-    pv2 = pv->ob_type->tp_as_number->nb_oct(pv);
+    /*
+     * If we still got an array which can hold references, stop
+     * because it could point back at 'v'.
+     */
+    if (PyArray_Check(pv) &&
+                    PyDataType_REFCHK(PyArray_DESCR((PyArrayObject *)pv))) {
+        PyErr_SetString(PyExc_TypeError,
+                "object array may be self-referencing");
+        return NULL;
+    }
+    pv2 = Py_TYPE(pv)->tp_as_number->nb_oct(pv);
     Py_DECREF(pv);
     return pv2;
 }
@@ -700,29 +824,40 @@ array_hex(PyArrayObject *v)
                         "be converted to Python scalars");
         return NULL;
     }
-    pv = v->descr->f->getitem(v->data, v);
-    if (pv->ob_type->tp_as_number == 0) {
+    pv = PyArray_DESCR(v)->f->getitem(PyArray_DATA(v), v);
+    if (Py_TYPE(pv)->tp_as_number == 0) {
         PyErr_SetString(PyExc_TypeError, "cannot convert to an int; "\
                         "scalar object is not a number");
         return NULL;
     }
-    if (pv->ob_type->tp_as_number->nb_hex == 0) {
+    if (Py_TYPE(pv)->tp_as_number->nb_hex == 0) {
         PyErr_SetString(PyExc_TypeError, "don't know how to convert "\
                         "scalar number to hex");
         return NULL;
     }
-    pv2 = pv->ob_type->tp_as_number->nb_hex(pv);
+    /*
+     * If we still got an array which can hold references, stop
+     * because it could point back at 'v'.
+     */
+    if (PyArray_Check(pv) &&
+                    PyDataType_REFCHK(PyArray_DESCR((PyArrayObject *)pv))) {
+        PyErr_SetString(PyExc_TypeError,
+                "object array may be self-referencing");
+        return NULL;
+    }
+    pv2 = Py_TYPE(pv)->tp_as_number->nb_hex(pv);
     Py_DECREF(pv);
     return pv2;
 }
 
+#endif
+
 static PyObject *
 _array_copy_nice(PyArrayObject *self)
 {
     return PyArray_Return((PyArrayObject *) PyArray_Copy(self));
 }
 
-#if PY_VERSION_HEX >= 0x02050000
 static PyObject *
 array_index(PyArrayObject *v)
 {
@@ -731,16 +866,23 @@ array_index(PyArrayObject *v)
                         "one element can be converted to an index");
         return NULL;
     }
-    return v->descr->f->getitem(v->data, v);
+    if (PyArray_NDIM(v) != 0) {
+        if (DEPRECATE("converting an array with ndim > 0 to an index"
+                      " will result in an error in the future") < 0) {
+            return NULL;
+        }
+    }
+    return PyArray_DESCR(v)->f->getitem(PyArray_DATA(v), v);
 }
-#endif
 
 
 NPY_NO_EXPORT PyNumberMethods array_as_number = {
     (binaryfunc)array_add,                      /*nb_add*/
     (binaryfunc)array_subtract,                 /*nb_subtract*/
     (binaryfunc)array_multiply,                 /*nb_multiply*/
+#if !defined(NPY_PY3K)
     (binaryfunc)array_divide,                   /*nb_divide*/
+#endif
     (binaryfunc)array_remainder,                /*nb_remainder*/
     (binaryfunc)array_divmod,                   /*nb_divmod*/
     (ternaryfunc)array_power,                   /*nb_power*/
@@ -754,12 +896,20 @@ NPY_NO_EXPORT PyNumberMethods array_as_number = {
     (binaryfunc)array_bitwise_and,              /*nb_and*/
     (binaryfunc)array_bitwise_xor,              /*nb_xor*/
     (binaryfunc)array_bitwise_or,               /*nb_or*/
+#if !defined(NPY_PY3K)
     0,                                          /*nb_coerce*/
+#endif
     (unaryfunc)array_int,                       /*nb_int*/
+#if defined(NPY_PY3K)
+    0,                                          /*nb_reserved*/
+#else
     (unaryfunc)array_long,                      /*nb_long*/
+#endif
     (unaryfunc)array_float,                     /*nb_float*/
+#if !defined(NPY_PY3K)
     (unaryfunc)array_oct,                       /*nb_oct*/
     (unaryfunc)array_hex,                       /*nb_hex*/
+#endif
 
     /*
      * This code adds augmented assignment functionality
@@ -768,7 +918,9 @@ NPY_NO_EXPORT PyNumberMethods array_as_number = {
     (binaryfunc)array_inplace_add,              /*inplace_add*/
     (binaryfunc)array_inplace_subtract,         /*inplace_subtract*/
     (binaryfunc)array_inplace_multiply,         /*inplace_multiply*/
+#if !defined(NPY_PY3K)
     (binaryfunc)array_inplace_divide,           /*inplace_divide*/
+#endif
     (binaryfunc)array_inplace_remainder,        /*inplace_remainder*/
     (ternaryfunc)array_inplace_power,           /*inplace_power*/
     (binaryfunc)array_inplace_left_shift,       /*inplace_lshift*/
@@ -781,9 +933,5 @@ NPY_NO_EXPORT PyNumberMethods array_as_number = {
     (binaryfunc)array_true_divide,              /*nb_true_divide*/
     (binaryfunc)array_inplace_floor_divide,     /*nb_inplace_floor_divide*/
     (binaryfunc)array_inplace_true_divide,      /*nb_inplace_true_divide*/
-
-#if PY_VERSION_HEX >= 0x02050000
     (unaryfunc)array_index,                     /* nb_index */
-#endif
-
 };
