@@ -4,25 +4,22 @@
 #include <locale.h>
 #include <stdio.h>
 
+#define NPY_NO_DEPRECATED_API NPY_API_VERSION
 #define _MULTIARRAYMODULE
-#define NPY_NO_PREFIX
 #include "numpy/arrayobject.h"
 #include "numpy/npy_math.h"
 
 #include "npy_config.h"
 
+#include "npy_pycompat.h"
+
 /*
  * From the C99 standard, section 7.19.6: The exponent always contains at least
  * two digits, and only as many more digits as necessary to represent the
  * exponent.
  */
 
-/* We force 3 digits on windows for python < 2.6 for compatibility reason */
-#if defined(MS_WIN32) && (PY_VERSION_HEX < 0x02060000)
-#define MIN_EXPONENT_DIGITS 3
-#else
 #define MIN_EXPONENT_DIGITS 2
-#endif
 
 /*
  * Ensure that any exponent, if present, is at least MIN_EXPONENT_DIGITS
@@ -334,7 +331,7 @@ _ASCII_FORMAT(long double, l, double)
  *
  * Same as isspace under C locale
  */
-static int
+NPY_NO_EXPORT int
 NumPyOS_ascii_isspace(char c)
 {
     return c == ' ' || c == '\f' || c == '\n' || c == '\r' || c == '\t'
@@ -418,6 +415,32 @@ NumPyOS_ascii_strncasecmp(const char* s1, const char* s2, size_t len)
     return 0;
 }
 
+/*
+ * _NumPyOS_ascii_strtod_plain:
+ *
+ * PyOS_ascii_strtod work-alike, with no enhanced features,
+ * for forward compatibility with Python >= 2.7
+ */
+static double
+NumPyOS_ascii_strtod_plain(const char *s, char** endptr)
+{
+    double result;
+#if PY_VERSION_HEX >= 0x02070000
+    NPY_ALLOW_C_API_DEF;
+    NPY_ALLOW_C_API;
+    result = PyOS_string_to_double(s, endptr, NULL);
+    if (PyErr_Occurred()) {
+        if (endptr) {
+            *endptr = (char*)s;
+        }
+        PyErr_Clear();
+    }
+    NPY_DISABLE_C_API;
+#else
+    result = PyOS_ascii_strtod(s, endptr);
+#endif
+    return result;
+}
 
 /*
  * NumPyOS_ascii_strtod:
@@ -506,7 +529,7 @@ NumPyOS_ascii_strtod(const char *s, char** endptr)
             }
             memcpy(buffer, s, n);
             buffer[n] = '\0';
-            result = PyOS_ascii_strtod(buffer, &q);
+            result = NumPyOS_ascii_strtod_plain(buffer, &q);
             if (endptr != NULL) {
                 *endptr = (char*)(s + (q - buffer));
             }
@@ -515,7 +538,7 @@ NumPyOS_ascii_strtod(const char *s, char** endptr)
     }
     /* End of ##2 */
 
-    return PyOS_ascii_strtod(s, endptr);
+    return NumPyOS_ascii_strtod_plain(s, endptr);
 }
 
 
