@@ -1,3 +1,5 @@
+#define NPY_NO_DEPRECATED_API NPY_API_VERSION
+
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
 
@@ -5,35 +7,36 @@
 #include <stdio.h>
 
 #define _MULTIARRAYMODULE
-#define NPY_NO_PREFIX
 #include "numpy/arrayobject.h"
 #include "numpy/npy_math.h"
 
 #include "npy_config.h"
 
-#include "numpy/npy_3kcompat.h"
+#include "npy_pycompat.h"
+#include "ctors.h"
 
-/* Functions only needed on narrow builds of Python
-   for converting back and forth between the NumPy Unicode data-type
-   (always 4-byte)
-   and the Python Unicode scalar (2-bytes on a narrow build).
-*/
+/*
+ * Functions only needed on narrow builds of Python for converting back and
+ * forth between the NumPy Unicode data-type (always 4-bytes) and the
+ * Python Unicode scalar (2-bytes on a narrow build).
+ */
 
-/* the ucs2 buffer must be large enough to hold 2*ucs4length characters
-   due to the use of surrogate pairs.
-
-   The return value is the number of ucs2 bytes used-up which
-   is ucs4length + number of surrogate pairs found.
-
-   values above 0xffff are converted to surrogate pairs.
-*/
+/*
+ * The ucs2 buffer must be large enough to hold 2*ucs4length characters
+ * due to the use of surrogate pairs.
+ *
+ * The return value is the number of ucs2 bytes used-up which
+ * is ucs4length + number of surrogate pairs found.
+ *
+ * Values above 0xffff are converted to surrogate pairs.
+ */
 NPY_NO_EXPORT int
-PyUCS2Buffer_FromUCS4(Py_UNICODE *ucs2, PyArray_UCS4 *ucs4, int ucs4length)
+PyUCS2Buffer_FromUCS4(Py_UNICODE *ucs2, npy_ucs4 *ucs4, int ucs4length)
 {
     int i;
     int numucs2 = 0;
-    PyArray_UCS4 chr;
-    for (i=0; i<ucs4length; i++) {
+    npy_ucs4 chr;
+    for (i = 0; i < ucs4length; i++) {
         chr = *ucs4++;
         if (chr > 0xffff) {
             numucs2++;
@@ -50,33 +53,33 @@ PyUCS2Buffer_FromUCS4(Py_UNICODE *ucs2, PyArray_UCS4 *ucs4, int ucs4length)
 }
 
 
-/* This converts a UCS2 buffer of the given length to UCS4 buffer.
-   It converts up to ucs4len characters of UCS2
-
-   It returns the number of characters converted which can
-   be less than ucs2len if there are surrogate pairs in ucs2.
-
-   The return value is the actual size of the used part of the ucs4 buffer.
-*/
-
+/*
+ * This converts a UCS2 buffer of the given length to UCS4 buffer.
+ * It converts up to ucs4len characters of UCS2
+ *
+ * It returns the number of characters converted which can
+ * be less than ucs2len if there are surrogate pairs in ucs2.
+ *
+ * The return value is the actual size of the used part of the ucs4 buffer.
+ */
 NPY_NO_EXPORT int
-PyUCS2Buffer_AsUCS4(Py_UNICODE *ucs2, PyArray_UCS4 *ucs4, int ucs2len, int ucs4len)
+PyUCS2Buffer_AsUCS4(Py_UNICODE *ucs2, npy_ucs4 *ucs4, int ucs2len, int ucs4len)
 {
     int i;
-    PyArray_UCS4 chr;
+    npy_ucs4 chr;
     Py_UNICODE ch;
     int numchars=0;
 
-    for (i=0; (i < ucs2len) && (numchars < ucs4len); i++) {
+    for (i = 0; (i < ucs2len) && (numchars < ucs4len); i++) {
         ch = *ucs2++;
         if (ch >= 0xd800 && ch <= 0xdfff) {
             /* surrogate pair */
-            chr = ((PyArray_UCS4)(ch-0xd800)) << 10;
+            chr = ((npy_ucs4)(ch-0xd800)) << 10;
             chr += *ucs2++ + 0x2400;  /* -0xdc00 + 0x10000 */
             i++;
         }
         else {
-            chr = (PyArray_UCS4) ch;
+            chr = (npy_ucs4) ch;
         }
         *ucs4++ = chr;
         numchars++;
