@@ -1,21 +1,26 @@
-from tempfile import NamedTemporaryFile, mktemp
+from __future__ import division, absolute_import, print_function
+
+import sys
+from tempfile import NamedTemporaryFile, TemporaryFile, mktemp, mkdtemp
 import os
-import warnings
+import shutil
 
 from numpy import memmap
-from numpy import arange, allclose
-from numpy.testing import TestCase, assert_, assert_array_equal
+from numpy import arange, allclose, asarray
+from numpy.testing import *
 
 class TestMemmap(TestCase):
     def setUp(self):
         self.tmpfp = NamedTemporaryFile(prefix='mmap')
-        self.shape = (3,4)
+        self.tempdir = mkdtemp()
+        self.shape = (3, 4)
         self.dtype = 'float32'
         self.data = arange(12, dtype=self.dtype)
         self.data.resize(self.shape)
 
     def tearDown(self):
         self.tmpfp.close()
+        shutil.rmtree(self.tempdir)
 
     def test_roundtrip(self):
         # Write data to file
@@ -31,61 +36,92 @@ class TestMemmap(TestCase):
         assert_array_equal(self.data, newfp)
 
     def test_open_with_filename(self):
-        tmpname = mktemp('','mmap')
+        tmpname = mktemp('', 'mmap', dir=self.tempdir)
         fp = memmap(tmpname, dtype=self.dtype, mode='w+',
                        shape=self.shape)
         fp[:] = self.data[:]
         del fp
-        os.unlink(tmpname)
+
+    def test_unnamed_file(self):
+        with TemporaryFile() as f:
+            fp = memmap(f, dtype=self.dtype, shape=self.shape)
+            del fp
 
     def test_attributes(self):
         offset = 1
         mode = "w+"
         fp = memmap(self.tmpfp, dtype=self.dtype, mode=mode,
                     shape=self.shape, offset=offset)
-        self.assertEquals(offset, fp.offset)
-        self.assertEquals(mode, fp.mode)
+        self.assertEqual(offset, fp.offset)
+        self.assertEqual(mode, fp.mode)
         del fp
 
     def test_filename(self):
-        tmpname = mktemp('','mmap')
+        tmpname = mktemp('', 'mmap', dir=self.tempdir)
         fp = memmap(tmpname, dtype=self.dtype, mode='w+',
                        shape=self.shape)
         abspath = os.path.abspath(tmpname)
         fp[:] = self.data[:]
-        self.assertEquals(abspath, fp.filename)
+        self.assertEqual(abspath, fp.filename)
         b = fp[:1]
-        self.assertEquals(abspath, b.filename)
+        self.assertEqual(abspath, b.filename)
         del b
         del fp
-        os.unlink(tmpname)
 
     def test_filename_fileobj(self):
         fp = memmap(self.tmpfp, dtype=self.dtype, mode="w+",
                     shape=self.shape)
-        self.assertEquals(fp.filename, self.tmpfp.name)
+        self.assertEqual(fp.filename, self.tmpfp.name)
 
+    @dec.knownfailureif(sys.platform=='gnu0', "This test is known to fail on hurd")
     def test_flush(self):
         fp = memmap(self.tmpfp, dtype=self.dtype, mode='w+',
                     shape=self.shape)
         fp[:] = self.data[:]
+        assert_equal(fp[0], self.data[0])
         fp.flush()
 
     def test_del(self):
         # Make sure a view does not delete the underlying mmap
         fp_base = memmap(self.tmpfp, dtype=self.dtype, mode='w+',
                     shape=self.shape)
-        fp_view = fp_base[:]
-        class ViewCloseError(Exception):
-            pass
-        _close = memmap._close
-        def replace_close(self):
-            raise ViewCloseError('View should not call _close on memmap')
-        try:
-            memmap._close = replace_close
-            del fp_view
-        finally:
-            memmap._close = _close
+        fp_base[0] = 5
+        fp_view = fp_base[0:1]
+        assert_equal(fp_view[0], 5)
+        del fp_view
+        # Should still be able to access and assign values after
+        # deleting the view
+        assert_equal(fp_base[0], 5)
+        fp_base[0] = 6
+        assert_equal(fp_base[0], 6)
+
+    def test_arithmetic_drops_references(self):
+        fp = memmap(self.tmpfp, dtype=self.dtype, mode='w+',
+                    shape=self.shape)
+        tmp = (fp + 10)
+        if isinstance(tmp, memmap):
+            assert tmp._mmap is not fp._mmap
+
+    def test_indexing_drops_references(self):
+        fp = memmap(self.tmpfp, dtype=self.dtype, mode='w+',
+                    shape=self.shape)
+        tmp = fp[[(1, 2), (2, 3)]]
+        if isinstance(tmp, memmap):
+            assert tmp._mmap is not fp._mmap
+
+    def test_slicing_keeps_references(self):
+        fp = memmap(self.tmpfp, dtype=self.dtype, mode='w+',
+                    shape=self.shape)
+        assert fp[:2, :2]._mmap is fp._mmap
+
+    def test_view(self):
+        fp = memmap(self.tmpfp, dtype=self.dtype, shape=self.shape)
+        new1 = fp.view()
+        new2 = new1.view()
+        assert(new1.base is fp)
+        assert(new2.base is fp)
+        new_array = asarray(fp)
+        assert(new_array.base is fp)
 
 if __name__ == "__main__":
     run_module_suite()
