@@ -1,3 +1,8 @@
+from __future__ import division, absolute_import, print_function
+
+import sys
+import platform
+
 from numpy.testing import *
 import numpy.core.umath as ncu
 import numpy as np
@@ -6,6 +11,22 @@ import numpy as np
 # TODO: conj 'symmetry'
 # TODO: FPU exceptions
 
+# At least on Windows the results of many complex functions are not conforming
+# to the C99 standard. See ticket 1574.
+# Ditto for Solaris (ticket 1642) and OS X on PowerPC.
+with np.errstate(all='ignore'):
+    functions_seem_flaky = ((np.exp(complex(np.inf, 0)).imag != 0)
+                            or (np.log(complex(np.NZERO, 0)).imag != np.pi))
+# TODO: replace with a check on whether platform-provided C99 funcs are used
+skip_complex_tests = (not sys.platform.startswith('linux') or functions_seem_flaky)
+
+def platform_skip(func):
+    return dec.skipif(skip_complex_tests,
+        "Numpy is using complex functions (e.g. sqrt) provided by your"
+        "platform's C library. However, they do not seem to behave according"
+        "to C99 -- so C99 tests are skipped.")(func)
+
+
 class TestCexp(object):
     def test_simple(self):
         check = check_complex_value
@@ -17,6 +38,7 @@ class TestCexp(object):
         ref = np.exp(1) * np.complex(np.cos(1), np.sin(1))
         yield check, f, 1, 1, ref.real, ref.imag, False
 
+    @platform_skip
     def test_special_values(self):
         # C99: Section G 6.3.1
 
@@ -52,38 +74,42 @@ class TestCexp(object):
 
         # cexp(-inf + inf i) is +-0 +- 0i (signs unspecified)
         def _check_ninf_inf(dummy):
-            z = f(np.array(np.complex(-np.inf, np.inf)))
-            if z.real != 0 or z.imag != 0:
-                raise AssertionError(
-                        "cexp(-inf, inf) is (%f, %f), expected (+-0, +-0)" \
-                        % (z.real, z.imag))
+            msgform = "cexp(-inf, inf) is (%f, %f), expected (+-0, +-0)"
+            with np.errstate(invalid='ignore'):
+                z = f(np.array(np.complex(-np.inf, np.inf)))
+                if z.real != 0 or z.imag != 0:
+                    raise AssertionError(msgform %(z.real, z.imag))
+
         yield _check_ninf_inf, None
 
         # cexp(inf + inf i) is +-inf + NaNi and raised invalid FPU ex.
         def _check_inf_inf(dummy):
-            z = f(np.array(np.complex(np.inf, np.inf)))
-            if not np.isinf(z.real) or not np.isnan(z.imag):
-                raise AssertionError(
-                        "cexp(inf, inf) is (%f, %f), expected (+-inf, nan)" \
-                        % (z.real, z.imag))
+            msgform = "cexp(inf, inf) is (%f, %f), expected (+-inf, nan)"
+            with np.errstate(invalid='ignore'):
+                z = f(np.array(np.complex(np.inf, np.inf)))
+                if not np.isinf(z.real) or not np.isnan(z.imag):
+                    raise AssertionError(msgform % (z.real, z.imag))
+
         yield _check_inf_inf, None
 
         # cexp(-inf + nan i) is +-0 +- 0i
         def _check_ninf_nan(dummy):
-            z = f(np.array(np.complex(-np.inf, np.nan)))
-            if z.real != 0 or z.imag != 0:
-                raise AssertionError(
-                        "cexp(-inf, nan) is (%f, %f), expected (+-0, +-0)" \
-                        % (z.real, z.imag))
+            msgform = "cexp(-inf, nan) is (%f, %f), expected (+-0, +-0)"
+            with np.errstate(invalid='ignore'):
+                z = f(np.array(np.complex(-np.inf, np.nan)))
+                if z.real != 0 or z.imag != 0:
+                    raise AssertionError(msgform % (z.real, z.imag))
+
         yield _check_ninf_nan, None
 
         # cexp(inf + nan i) is +-inf + nan
         def _check_inf_nan(dummy):
-            z = f(np.array(np.complex(np.inf, np.nan)))
-            if not np.isinf(z.real) or not np.isnan(z.imag):
-                raise AssertionError(
-                        "cexp(-inf, nan) is (%f, %f), expected (+-inf, nan)" \
-                        % (z.real, z.imag))
+            msgform = "cexp(-inf, nan) is (%f, %f), expected (+-inf, nan)"
+            with np.errstate(invalid='ignore'):
+                z = f(np.array(np.complex(np.inf, np.nan)))
+                if not np.isinf(z.real) or not np.isnan(z.imag):
+                    raise AssertionError(msgform % (z.real, z.imag))
+
         yield _check_inf_nan, None
 
         # cexp(nan + yi) is nan + nani for y != 0 (optional: raises invalid FPU
@@ -111,27 +137,37 @@ class TestClog(TestCase):
         for i in range(len(x)):
             assert_almost_equal(y[i], y_r[i])
 
-    @dec.knownfailureif(True, "clog(- inf + i inf) fails on Windows.")
+    @platform_skip
+    @dec.skipif(platform.machine() == "armv5tel", "See gh-413.")
     def test_special_values(self):
         xl = []
         yl = []
 
         # From C99 std (Sec 6.3.2)
         # XXX: check exceptions raised
+        # --- raise for invalid fails.
 
         # clog(-0 + i0) returns -inf + i pi and raises the 'divide-by-zero'
         # floating-point exception.
-        x = np.array([np.NZERO], dtype=np.complex)
-        y = np.complex(-np.inf, np.pi)
-        assert_almost_equal(np.log(x), y)
+        with np.errstate(divide='raise'):
+            x = np.array([np.NZERO], dtype=np.complex)
+            y = np.complex(-np.inf, np.pi)
+            self.assertRaises(FloatingPointError, np.log, x)
+        with np.errstate(divide='ignore'):
+            assert_almost_equal(np.log(x), y)
+
         xl.append(x)
         yl.append(y)
 
         # clog(+0 + i0) returns -inf + i0 and raises the 'divide-by-zero'
         # floating-point exception.
-        x = np.array([0], dtype=np.complex)
-        y = np.complex(-np.inf, 0)
-        assert_almost_equal(np.log(x), y)
+        with np.errstate(divide='raise'):
+            x = np.array([0], dtype=np.complex)
+            y = np.complex(-np.inf, 0)
+            self.assertRaises(FloatingPointError, np.log, x)
+        with np.errstate(divide='ignore'):
+            assert_almost_equal(np.log(x), y)
+
         xl.append(x)
         yl.append(y)
 
@@ -149,14 +185,22 @@ class TestClog(TestCase):
 
         # clog(x + iNaN) returns NaN + iNaN and optionally raises the
         # 'invalid' floating- point exception, for finite x.
-        x = np.array([complex(1., np.nan)], dtype=np.complex)
-        y = np.complex(np.nan, np.nan)
-        assert_almost_equal(np.log(x), y)
+        with np.errstate(invalid='raise'):
+            x = np.array([complex(1., np.nan)], dtype=np.complex)
+            y = np.complex(np.nan, np.nan)
+            #self.assertRaises(FloatingPointError, np.log, x)
+        with np.errstate(invalid='ignore'):
+            assert_almost_equal(np.log(x), y)
+
         xl.append(x)
         yl.append(y)
 
-        x = np.array([np.inf + np.nan * 1j], dtype=np.complex)
-        assert_almost_equal(np.log(x), y)
+        with np.errstate(invalid='raise'):
+            x = np.array([np.inf + 1j * np.nan], dtype=np.complex)
+            #self.assertRaises(FloatingPointError, np.log, x)
+        with np.errstate(invalid='ignore'):
+            assert_almost_equal(np.log(x), y)
+
         xl.append(x)
         yl.append(y)
 
@@ -225,10 +269,12 @@ class TestClog(TestCase):
         # clog(conj(z)) = conj(clog(z)).
         xa = np.array(xl, dtype=np.complex)
         ya = np.array(yl, dtype=np.complex)
-        for i in range(len(xa)):
-            assert_almost_equal(np.log(np.conj(xa[i])), np.conj(np.log(xa[i])))
+        with np.errstate(divide='ignore'):
+            for i in range(len(xa)):
+                assert_almost_equal(np.log(np.conj(xa[i])), np.conj(np.log(xa[i])))
 
 class TestCsqrt(object):
+
     def test_simple(self):
         # sqrt(1)
         yield check_complex_value, np.sqrt, 1, 0, 1, 0
@@ -248,6 +294,7 @@ class TestCsqrt(object):
     #def test_branch_cut(self):
     #    _check_branch_cut(f, -1, 0, 1, -1)
 
+    @platform_skip
     def test_special_values(self):
         check = check_complex_value
         f = np.sqrt
@@ -282,11 +329,12 @@ class TestCsqrt(object):
 
         # csqrt(-inf + nani) is nan +- infi (both +i infi are valid)
         def _check_ninf_nan(dummy):
+            msgform = "csqrt(-inf, nan) is (%f, %f), expected (nan, +-inf)"
             z = np.sqrt(np.array(np.complex(-np.inf, np.nan)))
-            if not np.isnan(z.real) or not np.isinf(z.imag):
-                raise AssertionError(
-                        "csqrt(-inf, nan) is (%f, %f), expected (nan, +-inf)" \
-                        % (z.real, z.imag))
+            #Fixme: ugly workaround for isinf bug.
+            with np.errstate(invalid='ignore'):
+                if not (np.isnan(z.real) and np.isinf(z.imag)):
+                    raise AssertionError(msgform % (z.real, z.imag))
 
         yield _check_ninf_nan, None
 
@@ -303,6 +351,12 @@ class TestCsqrt(object):
         # cuts first)
 
 class TestCpow(TestCase):
+    def setUp(self):
+        self.olderr = np.seterr(invalid='ignore')
+
+    def tearDown(self):
+        np.seterr(**self.olderr)
+
     def test_simple(self):
         x = np.array([1+1j, 0+2j, 1+2j, np.inf, np.nan])
         y_r = x ** 2
@@ -313,7 +367,7 @@ class TestCpow(TestCase):
     def test_scalar(self):
         x = np.array([1, 1j,         2,  2.5+.37j, np.inf, np.nan])
         y = np.array([1, 1j, -0.5+1.5j, -0.5+1.5j,      2,      3])
-        lx = range(len(x))
+        lx = list(range(len(x)))
         # Compute the values for complex type in python
         p_r = [complex(x[i]) ** complex(y[i]) for i in lx]
         # Substitute a result allowed by C99 standard
@@ -326,7 +380,7 @@ class TestCpow(TestCase):
     def test_array(self):
         x = np.array([1, 1j,         2,  2.5+.37j, np.inf, np.nan])
         y = np.array([1, 1j, -0.5+1.5j, -0.5+1.5j,      2,      3])
-        lx = range(len(x))
+        lx = list(range(len(x)))
         # Compute the values for complex type in python
         p_r = [complex(x[i]) ** complex(y[i]) for i in lx]
         # Substitute a result allowed by C99 standard
@@ -337,6 +391,12 @@ class TestCpow(TestCase):
             assert_almost_equal(n_r[i], p_r[i], err_msg='Loop %d\n' % i)
 
 class TestCabs(object):
+    def setUp(self):
+        self.olderr = np.seterr(invalid='ignore')
+
+    def tearDown(self):
+        np.seterr(**self.olderr)
+
     def test_simple(self):
         x = np.array([1+1j, 0+2j, 1+2j, np.inf, np.nan])
         y_r = np.array([np.sqrt(2.), 2, np.sqrt(5), np.inf, np.nan])
@@ -387,7 +447,6 @@ class TestCabs(object):
             return np.abs(np.complex(a, b))
 
         xa = np.array(x, dtype=np.complex)
-        ya = np.array(x, dtype=np.complex)
         for i in range(len(xa)):
             ref = g(x[i], y[i])
             yield check_real_value, f, x[i], y[i], ref
@@ -425,7 +484,7 @@ class TestCarg(object):
 
         # carg(+- 0 + yi) returns -pi/2 for y < 0
         yield check_real_value, ncu._arg, np.PZERO, -1, 0.5 * np.pi, False
-        yield check_real_value, ncu._arg, np.NZERO, -1,-0.5 * np.pi, False
+        yield check_real_value, ncu._arg, np.NZERO, -1, -0.5 * np.pi, False
 
     #def test_branch_cuts(self):
     #    _check_branch_cut(ncu._arg, -1, 1j, -1, 1)
@@ -468,10 +527,11 @@ def check_real_value(f, x1, y1, x, exact=True):
 def check_complex_value(f, x1, y1, x2, y2, exact=True):
     z1 = np.array([complex(x1, y1)])
     z2 = np.complex(x2, y2)
-    if exact:
-        assert_equal(f(z1), z2)
-    else:
-        assert_almost_equal(f(z1), z2)
+    with np.errstate(invalid='ignore'):
+        if exact:
+            assert_equal(f(z1), z2)
+        else:
+            assert_almost_equal(f(z1), z2)
 
 if __name__ == "__main__":
     run_module_suite()
