@@ -1,62 +1,64 @@
-"""mrecords
+""":mod:`numpy.ma..mrecords`
 
-Defines the equivalent of recarrays for maskedarray.
-Masked arrays already support named fields, but masking works only by records.
-By comparison, mrecarrays support masking individual fields.
+Defines the equivalent of :class:`numpy.recarrays` for masked arrays,
+where fields can be accessed as attributes.
+Note that :class:`numpy.ma.MaskedArray` already supports structured datatypes
+and the masking of individual fields.
 
 :author: Pierre Gerard-Marchant
+
 """
-#TODO: We should make sure that no field is called '_mask','mask','_fieldmask',
-#TODO: ...or whatever restricted keywords.
-#TODO: An idea would be to no bother in the first place, and then rename the
-#TODO: invalid fields with a trailing underscore...
-#TODO: Maybe we could just overload the parser function ?
+from __future__ import division, absolute_import, print_function
+
+#!!!: * We should make sure that no field is called '_mask','mask','_fieldmask',
+#!!!:   or whatever restricted keywords.
+#!!!:   An idea would be to no bother in the first place, and then rename the
+#!!!:   invalid fields with a trailing underscore...
+#!!!:   Maybe we could just overload the parser function ?
 
 
 __author__ = "Pierre GF Gerard-Marchant"
 
 import sys
-import types
+import warnings
 
 import numpy as np
-from numpy import bool_, complex_, float_, int_, str_, object_, dtype, \
-    chararray, ndarray, recarray, record, array as narray
 import numpy.core.numerictypes as ntypes
-from numpy.core.records import find_duplicate, format_parser
-from numpy.core.records import fromarrays as recfromarrays, \
-    fromrecords as recfromrecords
+from numpy.compat import basestring
+from numpy import (
+        bool_, dtype, ndarray, recarray, array as narray
+        )
+from numpy.core.records import (
+        fromarrays as recfromarrays, fromrecords as recfromrecords
+        )
 
 _byteorderconv = np.core.records._byteorderconv
 _typestr = ntypes._typestr
 
 import numpy.ma as ma
-from numpy.ma import MAError, MaskedArray, masked, nomask, masked_array,\
-    make_mask, mask_or, getdata, getmask, getmaskarray, filled, \
-    default_fill_value, masked_print_option
+from numpy.ma import MAError, MaskedArray, masked, nomask, masked_array, \
+                     getdata, getmaskarray, filled
+
 _check_fill_value = ma.core._check_fill_value
 
-import warnings
 
-__all__ = ['MaskedRecords','mrecarray',
-           'fromarrays','fromrecords','fromtextfile','addfield',
+__all__ = ['MaskedRecords', 'mrecarray',
+           'fromarrays', 'fromrecords', 'fromtextfile', 'addfield',
            ]
 
-reserved_fields = ['_data','_mask','_fieldmask', 'dtype']
+reserved_fields = ['_data', '_mask', '_fieldmask', 'dtype']
 
 def _getformats(data):
     "Returns the formats of each array of arraylist as a comma-separated string."
-    if hasattr(data,'dtype'):
+    if hasattr(data, 'dtype'):
         return ",".join([desc[1] for desc in data.dtype.descr])
 
     formats = ''
     for obj in data:
         obj = np.asarray(obj)
-#        if not isinstance(obj, ndarray):
-##        if not isinstance(obj, ndarray):
-#            raise ValueError, "item in the array list must be an ndarray."
         formats += _typestr[obj.dtype.type]
         if issubclass(obj.dtype.type, ntypes.flexible):
-            formats += `obj.itemsize`
+            formats += repr(obj.itemsize)
         formats += ','
     return formats[:-1]
 
@@ -75,7 +77,7 @@ If the argument `names` is not None, updates the field names to valid names.
         elif isinstance(names, str):
             new_names = names.split(',')
         else:
-            raise NameError, "illegal input names %s" % `names`
+            raise NameError("illegal input names %s" % repr(names))
         nnames = len(new_names)
         if nnames < ndescr:
             new_names += default_names[nnames:]
@@ -83,18 +85,18 @@ If the argument `names` is not None, updates the field names to valid names.
     for (n, d, t) in zip(new_names, default_names, descr.descr):
         if n in reserved_fields:
             if t[0] in reserved_fields:
-                ndescr.append((d,t[1]))
+                ndescr.append((d, t[1]))
             else:
                 ndescr.append(t)
         else:
-            ndescr.append((n,t[1]))
-    return numeric.dtype(ndescr)
+            ndescr.append((n, t[1]))
+    return np.dtype(ndescr)
 
 
 def _get_fieldmask(self):
-    mdescr = [(n,'|b1') for n in self.dtype.names]
+    mdescr = [(n, '|b1') for n in self.dtype.names]
     fdmask = np.empty(self.shape, dtype=mdescr)
-    fdmask.flat = tuple([False]*len(mdescr))
+    fdmask.flat = tuple([False] * len(mdescr))
     return fdmask
 
 
@@ -111,8 +113,6 @@ class MaskedRecords(MaskedArray, object):
     _fill_value : {record}
         Filling values for each field.
     """
-    _defaultfieldmask = nomask
-    _defaulthardmask = False
     #............................................
     def __new__(cls, shape, dtype=None, buf=None, offset=0, strides=None,
                 formats=None, names=None, titles=None,
@@ -122,14 +122,14 @@ class MaskedRecords(MaskedArray, object):
                 **options):
         #
         self = recarray.__new__(cls, shape, dtype=dtype, buf=buf, offset=offset,
-                                strides=strides, formats=formats,
-                                byteorder=byteorder, aligned=aligned,)
-#        self = self.view(cls)
+                                strides=strides, formats=formats, names=names,
+                                titles=titles, byteorder=byteorder,
+                                aligned=aligned,)
         #
-        mdtype = [(k,'|b1') for (k,_) in self.dtype.descr]
+        mdtype = ma.make_mask_descr(self.dtype)
         if mask is nomask or not np.size(mask):
             if not keep_mask:
-                self._fieldmask = tuple([False]*len(mdtype))
+                self._mask = tuple([False] * len(mdtype))
         else:
             mask = np.array(mask, copy=copy)
             if mask.shape != self.shape:
@@ -139,7 +139,7 @@ class MaskedRecords(MaskedArray, object):
                 elif nm == nd:
                     mask = np.reshape(mask, self.shape)
                 else:
-                    msg = "Mask and data not compatible: data size is %i, "+\
+                    msg = "Mask and data not compatible: data size is %i, " + \
                           "mask size is %i."
                     raise MAError(msg % (nd, nm))
                 copy = True
@@ -148,102 +148,44 @@ class MaskedRecords(MaskedArray, object):
                 self._sharedmask = True
             else:
                 if mask.dtype == mdtype:
-                    _fieldmask = mask
+                    _mask = mask
                 else:
-                    _fieldmask = np.array([tuple([m]*len(mdtype)) for m in mask],
-                                          dtype=mdtype)
-                self._fieldmask = _fieldmask
+                    _mask = np.array([tuple([m] * len(mdtype)) for m in mask],
+                                     dtype=mdtype)
+                self._mask = _mask
         return self
     #......................................................
-    def __array_finalize__(self,obj):
+    def __array_finalize__(self, obj):
         # Make sure we have a _fieldmask by default ..
-        _fieldmask = getattr(obj, '_fieldmask', None)
-        if _fieldmask is None:
-            mdescr = [(n,'|b1') for (n,_) in self.dtype.descr]
-            _mask = getattr(obj, '_mask', nomask)
-            if _mask is nomask:
-                _fieldmask = np.empty(self.shape, dtype=mdescr).view(recarray)
-                _fieldmask.flat = tuple([False]*len(mdescr))
+        _mask = getattr(obj, '_mask', None)
+        if _mask is None:
+            objmask = getattr(obj, '_mask', nomask)
+            _dtype = ndarray.__getattribute__(self, 'dtype')
+            if objmask is nomask:
+                _mask = ma.make_mask_none(self.shape, dtype=_dtype)
             else:
-                _fieldmask = narray([tuple([m]*len(mdescr)) for m in _mask],
-                                    dtype=mdescr).view(recarray)
+                mdescr = ma.make_mask_descr(_dtype)
+                _mask = narray([tuple([m] * len(mdescr)) for m in objmask],
+                               dtype=mdescr).view(recarray)
         # Update some of the attributes
-        if obj is not None:
-            _baseclass = getattr(obj,'_baseclass',type(obj))
-        else:
-            _baseclass = recarray
-        attrdict = dict(_fieldmask=_fieldmask,
-                        _hardmask=getattr(obj,'_hardmask',False),
-                        _fill_value=getattr(obj,'_fill_value',None),
-                        _sharedmask=getattr(obj,'_sharedmask',False),
-                        _baseclass=_baseclass)
-        self.__dict__.update(attrdict)
-        # Finalize as a regular maskedarray .....
-        # Update special attributes ...
-        self._basedict = getattr(obj, '_basedict', getattr(obj,'__dict__',{}))
-        self.__dict__.update(self._basedict)
+        _dict = self.__dict__
+        _dict.update(_mask=_mask)
+        self._update_from(obj)
+        if _dict['_baseclass'] == ndarray:
+            _dict['_baseclass'] = recarray
         return
-    #......................................................
+
+
     def _getdata(self):
         "Returns the data as a recarray."
-        return ndarray.view(self,recarray)
+        return ndarray.view(self, recarray)
     _data = property(fget=_getdata)
-    #......................................................
-    def __setmask__(self, mask):
-        "Sets the mask and update the fieldmask."
-        names = self.dtype.names
-        fmask = self.__dict__['_fieldmask']
-        #
-        if isinstance(mask,ndarray) and mask.dtype.names == names:
-            for n in names:
-                fmask[n] = mask[n].astype(bool)
-#            self.__dict__['_fieldmask'] = fmask.view(recarray)
-            return
-        newmask = make_mask(mask, copy=False)
-        if names is not None:
-            if self._hardmask:
-                for n in names:
-                    fmask[n].__ior__(newmask)
-            else:
-                for n in names:
-                    fmask[n].flat = newmask
-        return
-    _setmask = __setmask__
-    #
-    def _getmask(self):
-        """Return the mask of the mrecord.
-    A record is masked when all the fields are masked.
 
-        """
-        if self.size > 1:
-            return self._fieldmask.view((bool_, len(self.dtype))).all(1)
-        else:
-            return self._fieldmask.view((bool_, len(self.dtype))).all()
-    mask = _mask = property(fget=_getmask, fset=_setmask)
-    #......................................................
-    def get_fill_value(self):
-        """Return the filling value.
+    def _getfieldmask(self):
+        "Alias to mask"
+        return self._mask
+    _fieldmask = property(fget=_getfieldmask)
 
-        """
-        if self._fill_value is None:
-            ddtype = self.dtype
-            fillval = _check_fill_value(None, ddtype)
-            self._fill_value = np.array(tuple(fillval), dtype=ddtype)
-        return self._fill_value
-
-    def set_fill_value(self, value=None):
-        """Set the filling value to value.
-
-        If value is None, use a default based on the data type.
-
-        """
-        ddtype = self.dtype
-        fillval = _check_fill_value(value, ddtype)
-        self._fill_value = np.array(tuple(fillval), dtype=ddtype)
-
-    fill_value = property(fget=get_fill_value, fset=set_fill_value,
-                          doc="Filling value.")
-    #......................................................
     def __len__(self):
         "Returns the length"
         # We have more than one record
@@ -251,214 +193,213 @@ class MaskedRecords(MaskedArray, object):
             return len(self._data)
         # We have only one record: return the nb of fields
         return len(self.dtype)
-    #......................................................
+
     def __getattribute__(self, attr):
-        "Returns the given attribute."
         try:
-            # Returns a generic attribute
-            return object.__getattribute__(self,attr)
-        except AttributeError:
-            # OK, so attr must be a field name
+            return object.__getattribute__(self, attr)
+        except AttributeError: # attr must be a fieldname
             pass
-        # Get the list of fields ......
-        _names = self.dtype.names
-        if attr in _names:
-            _data = self._data
-            _mask = self._fieldmask
-#            obj = masked_array(_data.__getattribute__(attr), copy=False,
-#                               mask=_mask.__getattribute__(attr))
-            # Use a view in order to avoid the copy of the mask in MaskedArray.__new__
-            obj = narray(_data.__getattribute__(attr), copy=False).view(MaskedArray)
-            obj._mask = _mask.__getattribute__(attr)
-            if not obj.ndim and obj._mask:
-                return masked
-            return obj
-        raise AttributeError,"No attribute '%s' !" % attr
+        fielddict = ndarray.__getattribute__(self, 'dtype').fields
+        try:
+            res = fielddict[attr][:2]
+        except (TypeError, KeyError):
+            raise AttributeError("record array has no attribute %s" % attr)
+        # So far, so good...
+        _localdict = ndarray.__getattribute__(self, '__dict__')
+        _data = ndarray.view(self, _localdict['_baseclass'])
+        obj = _data.getfield(*res)
+        if obj.dtype.fields:
+            raise NotImplementedError("MaskedRecords is currently limited to"\
+                                      "simple records...")
+        # Get some special attributes
+        # Reset the object's mask
+        hasmasked = False
+        _mask = _localdict.get('_mask', None)
+        if _mask is not None:
+            try:
+                _mask = _mask[attr]
+            except IndexError:
+                # Couldn't find a mask: use the default (nomask)
+                pass
+            hasmasked = _mask.view((np.bool, (len(_mask.dtype) or 1))).any()
+        if (obj.shape or hasmasked):
+            obj = obj.view(MaskedArray)
+            obj._baseclass = ndarray
+            obj._isfield = True
+            obj._mask = _mask
+            # Reset the field values
+            _fill_value = _localdict.get('_fill_value', None)
+            if _fill_value is not None:
+                try:
+                    obj._fill_value = _fill_value[attr]
+                except ValueError:
+                    obj._fill_value = None
+        else:
+            obj = obj.item()
+        return obj
+
 
     def __setattr__(self, attr, val):
         "Sets the attribute attr to the value val."
-#        newattr = attr not in self.__dict__
+        # Should we call __setmask__ first ?
+        if attr in ['mask', 'fieldmask']:
+            self.__setmask__(val)
+            return
+        # Create a shortcut (so that we don't have to call getattr all the time)
+        _localdict = object.__getattribute__(self, '__dict__')
+        # Check whether we're creating a new field
+        newattr = attr not in _localdict
         try:
             # Is attr a generic attribute ?
             ret = object.__setattr__(self, attr, val)
         except:
             # Not a generic attribute: exit if it's not a valid field
-            fielddict = self.dtype.names or {}
-            if attr not in fielddict:
+            fielddict = ndarray.__getattribute__(self, 'dtype').fields or {}
+            optinfo = ndarray.__getattribute__(self, '_optinfo') or {}
+            if not (attr in fielddict or attr in optinfo):
                 exctype, value = sys.exc_info()[:2]
-                raise exctype, value
+                raise exctype(value)
         else:
-            if attr in ['_mask','fieldmask']:
-                self.__setmask__(val)
-                return
             # Get the list of names ......
-            _names = self.dtype.names
-            if _names is None:
-                _names = []
-            else:
-                _names = list(_names)
+            fielddict = ndarray.__getattribute__(self, 'dtype').fields or {}
             # Check the attribute
-            self_dict = self.__dict__
-            if attr not in _names+list(self_dict):
+            if attr not in fielddict:
                 return ret
-            if attr not in self_dict:         # We just added this one
+            if newattr:         # We just added this one
                 try:            #  or this setattr worked on an internal
                                 #  attribute.
                     object.__delattr__(self, attr)
                 except:
                     return ret
-        # Case #1.: Basic field ............
-        base_fmask = self._fieldmask
-        _names = self.dtype.names or []
-        if attr in _names:
-            if val is masked:
-                fval = self.fill_value[attr]
-                mval = True
+        # Let's try to set the field
+        try:
+            res = fielddict[attr][:2]
+        except (TypeError, KeyError):
+            raise AttributeError("record array has no attribute %s" % attr)
+        #
+        if val is masked:
+            _fill_value = _localdict['_fill_value']
+            if _fill_value is not None:
+                dval = _localdict['_fill_value'][attr]
             else:
-                fval = filled(val)
-                mval = getmaskarray(val)
-            if self._hardmask:
-                mval = mask_or(mval, base_fmask.__getattr__(attr))
-            self._data.__setattr__(attr, fval)
-            base_fmask.__setattr__(attr, mval)
-            return
-    #............................................
+                dval = val
+            mval = True
+        else:
+            dval = filled(val)
+            mval = getmaskarray(val)
+        obj = ndarray.__getattribute__(self, '_data').setfield(dval, *res)
+        _localdict['_mask'].__setitem__(attr, mval)
+        return obj
+
+
     def __getitem__(self, indx):
         """Returns all the fields sharing the same fieldname base.
 The fieldname base is either `_data` or `_mask`."""
         _localdict = self.__dict__
-        _fieldmask = _localdict['_fieldmask']
-        _data = self._data
+        _mask = ndarray.__getattribute__(self, '_mask')
+        _data = ndarray.view(self, _localdict['_baseclass'])
         # We want a field ........
         if isinstance(indx, basestring):
+            #!!!: Make sure _sharedmask is True to propagate back to _fieldmask
+            #!!!: Don't use _set_mask, there are some copies being made...
+            #!!!: ...that break propagation
+            #!!!: Don't force the mask to nomask, that wrecks easy masking
             obj = _data[indx].view(MaskedArray)
-            obj._set_mask(_fieldmask[indx])
-            # Force to nomask if the mask is empty
-            if not obj._mask.any():
-                obj._mask = nomask
+            obj._mask = _mask[indx]
+            obj._sharedmask = True
+            fval = _localdict['_fill_value']
+            if fval is not None:
+                obj._fill_value = fval[indx]
             # Force to masked if the mask is True
             if not obj.ndim and obj._mask:
                 return masked
             return obj
         # We want some elements ..
         # First, the data ........
-        obj = narray(_data[indx], copy=False).view(mrecarray)
-        obj._fieldmask = narray(_fieldmask[indx], copy=False).view(recarray)
+        obj = np.array(_data[indx], copy=False).view(mrecarray)
+        obj._mask = np.array(_mask[indx], copy=False).view(recarray)
         return obj
     #....
     def __setitem__(self, indx, value):
         "Sets the given record to value."
         MaskedArray.__setitem__(self, indx, value)
         if isinstance(indx, basestring):
-            self._fieldmask[indx] = ma.getmaskarray(value)
+            self._mask[indx] = ma.getmaskarray(value)
+
 
-    #............................................
-    def __setslice__(self, i, j, value):
-        "Sets the slice described by [i,j] to `value`."
-        _localdict = self.__dict__
-        d = self._data
-        m = _localdict['_fieldmask']
-        names = self.dtype.names
-        if value is masked:
-            for n in names:
-                m[i:j][n] = True
-        elif not self._hardmask:
-            fval = filled(value)
-            mval = getmaskarray(value)
-            for n in names:
-                d[n][i:j] = fval
-                m[n][i:j] = mval
-        else:
-            mindx = getmaskarray(self)[i:j]
-            dval = np.asarray(value)
-            valmask = getmask(value)
-            if valmask is nomask:
-                for n in names:
-                    mval = mask_or(m[n][i:j], valmask)
-                    d[n][i:j][~mval] = value
-            elif valmask.size > 1:
-                for n in names:
-                    mval = mask_or(m[n][i:j], valmask)
-                    d[n][i:j][~mval] = dval[~mval]
-                    m[n][i:j] = mask_or(m[n][i:j], mval)
-        self._fieldmask = m
-    #......................................................
     def __str__(self):
         "Calculates the string representation."
         if self.size > 1:
             mstr = ["(%s)" % ",".join([str(i) for i in s])
-                    for s in zip(*[getattr(self,f) for f in self.dtype.names])]
+                    for s in zip(*[getattr(self, f) for f in self.dtype.names])]
             return "[%s]" % ", ".join(mstr)
         else:
             mstr = ["%s" % ",".join([str(i) for i in s])
-                    for s in zip([getattr(self,f) for f in self.dtype.names])]
+                    for s in zip([getattr(self, f) for f in self.dtype.names])]
             return "(%s)" % ", ".join(mstr)
     #
     def __repr__(self):
         "Calculates the repr representation."
         _names = self.dtype.names
-        fmt = "%%%is : %%s" % (max([len(n) for n in _names])+4,)
-        reprstr = [fmt % (f,getattr(self,f)) for f in self.dtype.names]
-        reprstr.insert(0,'masked_records(')
+        fmt = "%%%is : %%s" % (max([len(n) for n in _names]) + 4,)
+        reprstr = [fmt % (f, getattr(self, f)) for f in self.dtype.names]
+        reprstr.insert(0, 'masked_records(')
         reprstr.extend([fmt % ('    fill_value', self.fill_value),
                          '              )'])
         return str("\n".join(reprstr))
-    #......................................................
-    def view(self, obj):
+#    #......................................................
+    def view(self, dtype=None, type=None):
         """Returns a view of the mrecarray."""
-        try:
-            if issubclass(obj, ndarray):
-                return ndarray.view(self, obj)
-        except TypeError:
-            pass
-        dtype = np.dtype(obj)
-        if dtype.fields is None:
-            return self.__array__().view(dtype)
-        return ndarray.view(self, obj)
-    #......................................................
-    def filled(self, fill_value=None):
-        """Returns an array of the same class as the _data part, where masked
-    values are filled with fill_value.
-    If fill_value is None, self.fill_value is used instead.
-
-    Subclassing is preserved.
-
-        """
-        _localdict = self.__dict__
-        d = self._data
-        fm = _localdict['_fieldmask']
-        if not np.asarray(fm, dtype=bool_).any():
-            return d
-        #
-        if fill_value is None:
-            value = _check_fill_value(_localdict['_fill_value'],self.dtype)
+        # OK, basic copy-paste from MaskedArray.view...
+        if dtype is None:
+            if type is None:
+                output = ndarray.view(self)
+            else:
+                output = ndarray.view(self, type)
+        # Here again...
+        elif type is None:
+            try:
+                if issubclass(dtype, ndarray):
+                    output = ndarray.view(self, dtype)
+                    dtype = None
+                else:
+                    output = ndarray.view(self, dtype)
+            # OK, there's the change
+            except TypeError:
+                dtype = np.dtype(dtype)
+                # we need to revert to MaskedArray, but keeping the possibility
+                # ...of subclasses (eg, TimeSeriesRecords), so we'll force a type
+                # ...set to the first parent
+                if dtype.fields is None:
+                    basetype = self.__class__.__bases__[0]
+                    output = self.__array__().view(dtype, basetype)
+                    output._update_from(self)
+                else:
+                    output = ndarray.view(self, dtype)
+                output._fill_value = None
         else:
-            value = fill_value
-            if np.size(value) == 1:
-                value = [value,] * len(self.dtype)
-        #
-        if self is masked:
-            result = np.asanyarray(value)
-        else:
-            result = d.copy()
-            for (n, v) in zip(d.dtype.names, value):
-                np.putmask(np.asarray(result[n]), np.asarray(fm[n]), v)
-        return result
-    #......................................................
+            output = ndarray.view(self, dtype, type)
+        # Update the mask, just like in MaskedArray.view
+        if (getattr(output, '_mask', nomask) is not nomask):
+            mdtype = ma.make_mask_descr(output.dtype)
+            output._mask = self._mask.view(mdtype, ndarray)
+            output._mask.shape = output.shape
+        return output
+
     def harden_mask(self):
         "Forces the mask to hard"
         self._hardmask = True
     def soften_mask(self):
         "Forces the mask to soft"
         self._hardmask = False
-    #......................................................
+
     def copy(self):
         """Returns a copy of the masked record."""
         _localdict = self.__dict__
         copied = self._data.copy().view(type(self))
-        copied._fieldmask = self._fieldmask.copy()
+        copied._mask = self._mask.copy()
         return copied
-    #......................................................
+
     def tolist(self, fill_value=None):
         """Copy the data portion of the array to a hierarchical python
         list and returns that list.
@@ -472,7 +413,7 @@ The fieldname base is either `_data` or `_mask`."""
         if fill_value is not None:
             return self.filled(fill_value).tolist()
         result = narray(self.filled().tolist(), dtype=object)
-        mask = narray(self._fieldmask.tolist())
+        mask = narray(self._mask.tolist())
         result[mask] = None
         return result.tolist()
     #--------------------------------------------
@@ -486,7 +427,7 @@ The fieldname base is either `_data` or `_mask`."""
                  self.dtype,
                  self.flags.fnc,
                  self._data.tostring(),
-                 self._fieldmask.tostring(),
+                 self._mask.tostring(),
                  self._fill_value,
                  )
         return state
@@ -505,8 +446,8 @@ The fieldname base is either `_data` or `_mask`."""
         """
         (ver, shp, typ, isf, raw, msk, flv) = state
         ndarray.__setstate__(self, (shp, typ, isf, raw))
-        mdtype = dtype([(k,bool_) for (k,_) in self.dtype.descr])
-        self.__dict__['_fieldmask'].__setstate__((shp, mdtype, isf, msk))
+        mdtype = dtype([(k, bool_) for (k, _) in self.dtype.descr])
+        self.__dict__['_mask'].__setstate__((shp, mdtype, isf, msk))
         self.fill_value = flv
     #
     def __reduce__(self):
@@ -514,7 +455,7 @@ The fieldname base is either `_data` or `_mask`."""
 
         """
         return (_mrreconstruct,
-                (self.__class__, self._baseclass, (0,), 'b', ),
+                (self.__class__, self._baseclass, (0,), 'b',),
                 self.__getstate__())
 
 def _mrreconstruct(subtype, baseclass, baseshape, basetype,):
@@ -570,7 +511,7 @@ def fromarrays(arraylist, dtype=None, shape=None, formats=None,
                            dtype=dtype, shape=shape, formats=formats,
                            names=names, titles=titles, aligned=aligned,
                            byteorder=byteorder).view(mrecarray)
-    _array._fieldmask.flat = zip(*masklist)
+    _array._mask.flat = list(zip(*masklist))
     if fill_value is not None:
         _array.fill_value = fill_value
     return _array
@@ -604,16 +545,20 @@ def fromrecords(reclist, dtype=None, shape=None, formats=None, names=None,
     mask : {nomask, sequence}, optional.
         External mask to apply on the data.
 
-*Notes*:
+    Notes
+    -----
     Lists of tuples should be preferred over lists of lists for faster processing.
     """
     # Grab the initial _fieldmask, if needed:
-    _fieldmask = getattr(reclist, '_fieldmask', None)
+    _mask = getattr(reclist, '_mask', None)
     # Get the list of records.....
-    nfields = len(reclist[0])
+    try:
+        nfields = len(reclist[0])
+    except TypeError:
+        nfields = len(reclist[0].dtype)
     if isinstance(reclist, ndarray):
         # Make sure we don't have some hidden mask
-        if isinstance(reclist,MaskedArray):
+        if isinstance(reclist, MaskedArray):
             reclist = reclist.filled().view(ndarray)
         # Grab the initial dtype, just in case
         if dtype is None:
@@ -630,13 +575,13 @@ def fromrecords(reclist, dtype=None, shape=None, formats=None, names=None,
         mask = np.array(mask, copy=False)
         maskrecordlength = len(mask.dtype)
         if maskrecordlength:
-            mrec._fieldmask.flat = mask
+            mrec._mask.flat = mask
         elif len(mask.shape) == 2:
-            mrec._fieldmask.flat = [tuple(m) for m in mask]
+            mrec._mask.flat = [tuple(m) for m in mask]
         else:
-            mrec._mask = mask
-    if _fieldmask is not None:
-        mrec._fieldmask[:] = _fieldmask
+            mrec.__setmask__(mask)
+    if _mask is not None:
+        mrec._mask[:] = _mask
     return mrec
 
 def _guessvartypes(arr):
@@ -650,25 +595,25 @@ on the first line. An exception is raised if the file is 3D or more.
     if len(arr.shape) == 2 :
         arr = arr[0]
     elif len(arr.shape) > 2:
-        raise ValueError, "The array should be 2D at most!"
+        raise ValueError("The array should be 2D at most!")
     # Start the conversion loop .......
     for f in arr:
         try:
-            val = int(f)
+            int(f)
         except ValueError:
             try:
-                val = float(f)
+                float(f)
             except ValueError:
                 try:
                     val = complex(f)
                 except ValueError:
                     vartypes.append(arr.dtype)
                 else:
-                    vartypes.append(complex)
+                    vartypes.append(np.dtype(complex))
             else:
-                vartypes.append(float)
+                vartypes.append(np.dtype(float))
         else:
-            vartypes.append(int)
+            vartypes.append(np.dtype(int))
     return vartypes
 
 def openfile(fname):
@@ -680,11 +625,12 @@ def openfile(fname):
     try:
         f = open(fname)
     except IOError:
-        raise IOError, "No such file: '%s'" % fname
+        raise IOError("No such file: '%s'" % fname)
     if f.readline()[:2] != "\\x":
-        f.seek(0,0)
+        f.seek(0, 0)
         return f
-    raise NotImplementedError, "Wow, binary file"
+    f.close()
+    raise NotImplementedError("Wow, binary file")
 
 
 def fromtextfile(fname, delimitor=None, commentchar='#', missingchar='',
@@ -713,6 +659,7 @@ def fromtextfile(fname, delimitor=None, commentchar='#', missingchar='',
     Ultra simple: the varnames are in the header, one line"""
     # Try to open the file ......................
     f = openfile(fname)
+
     # Get the first non-empty line as the varnames
     while True:
         line = f.readline()
@@ -722,10 +669,13 @@ def fromtextfile(fname, delimitor=None, commentchar='#', missingchar='',
             break
     if varnames is None:
         varnames = _varnames
+
     # Get the data ..............................
     _variables = masked_array([line.strip().split(delimitor) for line in f
                                   if line[0] != commentchar and len(line) > 1])
     (_, nfields) = _variables.shape
+    f.close()
+
     # Try to guess the dtype ....................
     if vartypes is None:
         vartypes = _guessvartypes(_variables[0])
@@ -736,13 +686,17 @@ def fromtextfile(fname, delimitor=None, commentchar='#', missingchar='',
             msg += " Reverting to default."
             warnings.warn(msg % (len(vartypes), nfields))
             vartypes = _guessvartypes(_variables[0])
+
     # Construct the descriptor ..................
-    mdescr = [(n,f) for (n,f) in zip(varnames, vartypes)]
+    mdescr = [(n, f) for (n, f) in zip(varnames, vartypes)]
+    mfillv = [ma.default_fill_value(f) for f in vartypes]
+
     # Get the data and the mask .................
     # We just need a list of masked_arrays. It's easier to create it like that:
     _mask = (_variables.T == missingchar)
-    _datalist = [masked_array(a,mask=m,dtype=t)
-                     for (a,m,t) in zip(_variables.T, _mask, vartypes)]
+    _datalist = [masked_array(a, mask=m, dtype=t, fill_value=f)
+                 for (a, m, t, f) in zip(_variables.T, _mask, vartypes, mfillv)]
+
     return fromarrays(_datalist, dtype=mdescr)
 
 #....................................................................
@@ -752,7 +706,7 @@ and `newfieldname` as name. If `newfieldname` is None, the new field name is
 set to 'fi', where `i` is the number of existing fields.
     """
     _data = mrecord._data
-    _mask = mrecord._fieldmask
+    _mask = mrecord._mask
     if newfieldname is None or newfieldname in reserved_fields:
         newfieldname = 'f%i' % len(_data.dtype)
     newfield = ma.array(newfield)
@@ -761,22 +715,20 @@ set to 'fi', where `i` is the number of existing fields.
     newdtype = np.dtype(_data.dtype.descr + [(newfieldname, newfield.dtype)])
     newdata = recarray(_data.shape, newdtype)
     # Add the exisintg field
-    [newdata.setfield(_data.getfield(*f),*f)
+    [newdata.setfield(_data.getfield(*f), *f)
          for f in _data.dtype.fields.values()]
     # Add the new field
     newdata.setfield(newfield._data, *newdata.dtype.fields[newfieldname])
     newdata = newdata.view(MaskedRecords)
     # Get the new mask .............
     # Create a new empty recarray
-    newmdtype = np.dtype([(n,bool_) for n in newdtype.names])
+    newmdtype = np.dtype([(n, bool_) for n in newdtype.names])
     newmask = recarray(_data.shape, newmdtype)
     # Add the old masks
-    [newmask.setfield(_mask.getfield(*f),*f)
+    [newmask.setfield(_mask.getfield(*f), *f)
          for f in _mask.dtype.fields.values()]
     # Add the mask of the new field
     newmask.setfield(getmaskarray(newfield),
                      *newmask.dtype.fields[newfieldname])
-    newdata._fieldmask = newmask
+    newdata._mask = newmask
     return newdata
-
-###############################################################################
