@@ -1,7 +1,9 @@
 #include <Python.h>
 
+#define NPY_NO_DEPRECATED_API NPY_API_VERSION
 #define _libnumarray_MODULE
-#include "numpy/libnumarray.h"
+#include "include/numpy/libnumarray.h"
+#include "numpy/npy_3kcompat.h"
 #include <float.h>
 
 #if (defined(__unix__) || defined(unix)) && !defined(USG)
@@ -55,7 +57,7 @@ getBuffer( PyObject *obj)
 {
     if (!obj) return PyErr_Format(PyExc_RuntimeError,
             "NULL object passed to getBuffer()");
-    if (obj->ob_type->tp_as_buffer == NULL) {
+    if (((PyObject*)obj)->ob_type->tp_as_buffer == NULL) {
         return PyObject_CallMethod(obj, "__buffer__", NULL);
     } else {
         Py_INCREF(obj);  /* Since CallMethod returns a new object when it
@@ -88,6 +90,12 @@ isBuffer (PyObject *obj)
 static int
 getWriteBufferDataPtr(PyObject *buffobj, void **buff)
 {
+#if defined(NPY_PY3K)
+    /* FIXME: XXX - needs implementation */
+    PyErr_SetString(PyExc_RuntimeError,
+                    "XXX: getWriteBufferDataPtr is not implemented");
+    return -1;
+#else
     int rval = -1;
     PyObject *buff2;
     if ((buff2 = getBuffer(buffobj)))
@@ -98,6 +106,7 @@ getWriteBufferDataPtr(PyObject *buffobj, void **buff)
         Py_DECREF(buff2);
     }
     return rval;
+#endif
 }
 
 /**********************************************************************/
@@ -118,6 +127,12 @@ isBufferWriteable (PyObject *buffobj)
 static int
 getReadBufferDataPtr(PyObject *buffobj, void **buff)
 {
+#if defined(NPY_PY3K)
+    /* FIXME: XXX - needs implementation */
+    PyErr_SetString(PyExc_RuntimeError,
+                    "XXX: getWriteBufferDataPtr is not implemented");
+    return -1;
+#else
     int rval = -1;
     PyObject *buff2;
     if ((buff2 = getBuffer(buffobj))) {
@@ -127,6 +142,7 @@ getReadBufferDataPtr(PyObject *buffobj, void **buff)
         Py_DECREF(buff2);
     }
     return rval;
+#endif
 }
 
 /**********************************************************************/
@@ -134,6 +150,12 @@ getReadBufferDataPtr(PyObject *buffobj, void **buff)
 static int
 getBufferSize(PyObject *buffobj)
 {
+#if defined(NPY_PY3K)
+    /* FIXME: XXX - needs implementation */
+    PyErr_SetString(PyExc_RuntimeError,
+                    "XXX: getWriteBufferDataPtr is not implemented");
+    return -1;
+#else
     Py_ssize_t size=0;
     PyObject *buff2;
     if ((buff2 = getBuffer(buffobj)))
@@ -144,6 +166,7 @@ getBufferSize(PyObject *buffobj)
     else
         size = -1;
     return size;
+#endif
 }
 
 
@@ -325,9 +348,9 @@ NA_NewAll(int ndim, maybelong *shape, NumarrayType type,
             result = NULL;
         } else {
             if (buffer) {
-                memcpy(result->data, buffer, NA_NBYTES(result));
+                memcpy(PyArray_DATA(result), buffer, NA_NBYTES(result));
             } else {
-                memset(result->data, 0, NA_NBYTES(result));
+                memset(PyArray_DATA(result), 0, NA_NBYTES(result));
             }
         }
     }
@@ -344,7 +367,7 @@ NA_NewAllStrides(int ndim, maybelong *shape, maybelong *strides,
             byteoffset, 0,
             byteorder, aligned, writeable);
     for(i=0; i<ndim; i++)
-        result->strides[i] = strides[i];
+        PyArray_STRIDES(result)[i] = strides[i];
     return result;
 }
 
@@ -681,16 +704,16 @@ _NA_callStridingHelper(PyObject *aux, long dim,
 {
     int i, j, status=0;
     dim -= 1;
-    for(i=0; i<numarray[0]->dimensions[dim]; i++) {
+    for(i=0; i<PyArray_DIMS(numarray[0])[dim]; i++) {
         for (j=0; j<nnumarray; j++)
-            data[j] += numarray[j]->strides[dim]*i;
+            data[j] += PyArray_STRIDES(numarray[j])[dim]*i;
         if (dim == 0)
             status |= f(aux, nnumarray, numarray, data);
         else
             status |= _NA_callStridingHelper(
                     aux, dim, nnumarray, numarray, data, f);
         for (j=0; j<nnumarray; j++)
-            data[j] -= numarray[j]->strides[dim]*i;
+            data[j] -= PyArray_STRIDES(numarray[j])[dim]*i;
     }
     return status;
 }
@@ -724,7 +747,7 @@ callStridingCFunc(PyObject *self, PyObject *args) {
                     "%s arg[%d] is not an array.",
                     me->descr.name, i);
         numarray[i] = (PyArrayObject *) otemp;
-        data[i] = numarray[i]->data;
+        data[i] = PyArray_DATA(numarray[i]);
         Py_DECREF(otemp);
         if (!NA_updateDataPtr(numarray[i]))
             return NULL;
@@ -733,7 +756,7 @@ callStridingCFunc(PyObject *self, PyObject *args) {
     /* Cast function pointer and perform stride operation */
     f = (CFUNC_STRIDED_FUNC) me->descr.fptr;
 
-    if (_NA_callStridingHelper(aux, numarray[0]->nd,
+    if (_NA_callStridingHelper(aux, PyArray_NDIM(numarray[0]),
                 nnumarray, numarray, data, f)) {
         return NULL;
     } else {
@@ -901,7 +924,7 @@ cfunc_call(PyObject *self, PyObject *argsTuple, PyObject *NPY_UNUSED(argsDict))
     }
 }
 
-staticforward PyTypeObject CfuncType;
+static PyTypeObject CfuncType;
 
 static void
 cfunc_dealloc(PyObject* self)
@@ -918,12 +941,16 @@ cfunc_repr(PyObject *self)
             me->descr.name, (unsigned long ) me->descr.fptr,
             me->descr.chkself, me->descr.align,
             me->descr.wantIn, me->descr.wantOut);
-    return PyString_FromString(buf);
+    return PyUString_FromString(buf);
 }
 
 static PyTypeObject CfuncType = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+#if defined(NPY_PY3K)
+    PyVarObject_HEAD_INIT(0,0)
+#else
+    PyObject_HEAD_INIT(0)
+    0,                                          /* ob_size */
+#endif
     "Cfunc",
     sizeof(CfuncObject),
     0,
@@ -969,9 +996,7 @@ static PyTypeObject CfuncType = {
     0,                                          /* tp_subclasses */
     0,                                          /* tp_weaklist */
     0,                                          /* tp_del */
-#if PY_VERSION_HEX >= 0x02060000
     0,                                          /* tp_version_tag */
-#endif
     };
 
 /* CfuncObjects are created at the c-level only.  They ensure that each
@@ -986,8 +1011,9 @@ NA_new_cfunc(CfuncDescriptor *cfd)
 {
     CfuncObject* cfunc;
 
-    CfuncType.ob_type = &PyType_Type;  /* Should be done once at init.
-                                          Do now since there is no init. */
+    /* Should be done once at init.
+       Do now since there is no init. */
+    ((PyObject*)&CfuncType)->ob_type = &PyType_Type;
 
     cfunc = PyObject_New(CfuncObject, &CfuncType);
 
@@ -1025,7 +1051,7 @@ NA_InputArray(PyObject *a, NumarrayType t, int requires)
 static int
 satisfies(PyArrayObject *a, int requirements, NumarrayType t)
 {
-    int type_ok = (a->descr->type_num == t) || (t == tAny);
+    int type_ok = (PyArray_DESCR(a)->type_num == t) || (t == tAny);
 
     if (PyArray_ISCARRAY(a))
         return type_ok;
@@ -1049,9 +1075,12 @@ NA_OutputArray(PyObject *a, NumarrayType t, int requires)
     PyArray_Descr *dtype;
     PyArrayObject *ret;
 
-    if (!PyArray_Check(a) || !PyArray_ISWRITEABLE(a)) {
+    if (!PyArray_Check(a)) {
         PyErr_Format(PyExc_TypeError,
-                "NA_OutputArray: only writeable arrays work for output.");
+                "NA_OutputArray: only arrays work for output.");
+        return NULL;
+    }
+    if (PyArray_FailUnlessWriteable((PyArrayObject *)a, "output array") < 0) {
         return NULL;
     }
 
@@ -1060,18 +1089,20 @@ NA_OutputArray(PyObject *a, NumarrayType t, int requires)
         return (PyArrayObject *)a;
     }
     if (t == tAny) {
-        dtype = PyArray_DESCR(a);
+        dtype = PyArray_DESCR((PyArrayObject *)a);
         Py_INCREF(dtype);
     }
     else {
         dtype = PyArray_DescrFromType(t);
     }
-    ret = (PyArrayObject *)PyArray_Empty(PyArray_NDIM(a), PyArray_DIMS(a),
-            dtype, 0);
-    ret->flags |= NPY_UPDATEIFCOPY;
-    ret->base = a;
-    PyArray_FLAGS(a) &= ~NPY_WRITEABLE;
+    ret = (PyArrayObject *)PyArray_Empty(PyArray_NDIM((PyArrayObject *)a),
+                                        PyArray_DIMS((PyArrayObject *)a),
+                                        dtype, 0);
     Py_INCREF(a);
+    if (PyArray_SetUpdateIfCopyBase(ret, a) < 0) {
+        Py_DECREF(ret);
+        return NULL;
+    }
     return ret;
 }
 
@@ -1087,16 +1118,15 @@ NA_OutputArray(PyObject *a, NumarrayType t, int requires)
 static PyArrayObject *
 NA_IoArray(PyObject *a, NumarrayType t, int requires)
 {
-    PyArrayObject *shadow = NA_InputArray(a, t, requires | NPY_UPDATEIFCOPY );
+    PyArrayObject *shadow = NA_InputArray(a, t,
+                                requires | NPY_ARRAY_UPDATEIFCOPY );
 
     if (!shadow) return NULL;
 
     /* Guard against non-writable, but otherwise satisfying requires.
        In this case,  shadow == a.
        */
-    if (!PyArray_ISWRITABLE(shadow)) {
-        PyErr_Format(PyExc_TypeError,
-                "NA_IoArray: I/O array must be writable array");
+    if (!PyArray_FailUnlessWriteable(shadow, "input/output array")) {
         PyArray_XDECREF_ERR(shadow);
         return NULL;
     }
@@ -1132,7 +1162,7 @@ Complex64 NA_get_Complex64(PyArrayObject *a, long offset)
     Complex32 v0;
     Complex64 v;
 
-    switch(a->descr->type_num) {
+    switch(PyArray_DESCR(a)->type_num) {
     case tComplex32:
         v0 = NA_GETP(a, Complex32, (NA_PTR(a)+offset));
         v.r = v0.r;
@@ -1153,7 +1183,7 @@ void NA_set_Complex64(PyArrayObject *a, long offset, Complex64 v)
 {
     Complex32 v0;
 
-    switch(a->descr->type_num) {
+    switch(PyArray_DESCR(a)->type_num) {
     case tComplex32:
         v0.r = v.r;
         v0.i = v.i;
@@ -1170,7 +1200,7 @@ void NA_set_Complex64(PyArrayObject *a, long offset, Complex64 v)
 
 Int64 NA_get_Int64(PyArrayObject *a, long offset)
 {
-    switch(a->descr->type_num) {
+    switch(PyArray_DESCR(a)->type_num) {
     case tBool:
         return NA_GETP(a, Bool, (NA_PTR(a)+offset)) != 0;
     case tInt8:
@@ -1200,7 +1230,7 @@ Int64 NA_get_Int64(PyArrayObject *a, long offset)
     default:
         PyErr_Format( PyExc_TypeError,
                 "Unknown type %d in NA_get_Int64",
-                a->descr->type_num);
+                PyArray_DESCR(a)->type_num);
         PyErr_Print();
     }
     return 0; /* suppress warning */
@@ -1210,7 +1240,7 @@ void NA_set_Int64(PyArrayObject *a, long offset, Int64 v)
 {
     Bool b;
 
-    switch(a->descr->type_num) {
+    switch(PyArray_DESCR(a)->type_num) {
     case tBool:
         b = (v != 0);
         NA_SETP(a, Bool, (NA_PTR(a)+offset), b);
@@ -1248,7 +1278,7 @@ void NA_set_Int64(PyArrayObject *a, long offset, Int64 v)
     default:
                     PyErr_Format( PyExc_TypeError,
                             "Unknown type %d in NA_set_Int64",
-                            a->descr->type_num);
+                            PyArray_DESCR(a)->type_num);
                     PyErr_Print();
     }
 }
@@ -1266,11 +1296,11 @@ long NA_get_offset(PyArrayObject *a, int N, ...)
     va_start(ap, N);
     if (N > 0) { /* compute offset of "outer" indices. */
         for(i=0; i<N; i++)
-            offset += va_arg(ap, long) * a->strides[i];
+            offset += va_arg(ap, long) * PyArray_STRIDES(a)[i];
     } else {   /* compute offset of "inner" indices. */
         N = -N;
         for(i=0; i<N; i++)
-            offset += va_arg(ap, long) * a->strides[a->nd-N+i];
+            offset += va_arg(ap, long) * PyArray_STRIDES(a)[PyArray_NDIM(a)-N+i];
     }
     va_end(ap);
     return offset;
@@ -1278,7 +1308,7 @@ long NA_get_offset(PyArrayObject *a, int N, ...)
 
 Float64 NA_get_Float64(PyArrayObject *a, long offset)
 {
-    switch(a->descr->type_num) {
+    switch(PyArray_DESCR(a)->type_num) {
     case tBool:
         return NA_GETP(a, Bool, (NA_PTR(a)+offset)) != 0;
     case tInt8:
@@ -1310,7 +1340,7 @@ Float64 NA_get_Float64(PyArrayObject *a, long offset)
     default:
         PyErr_Format( PyExc_TypeError,
                 "Unknown type %d in NA_get_Float64",
-                a->descr->type_num);
+                PyArray_DESCR(a)->type_num);
     }
     return 0; /* suppress warning */
 }
@@ -1319,7 +1349,7 @@ void NA_set_Float64(PyArrayObject *a, long offset, Float64 v)
 {
     Bool b;
 
-    switch(a->descr->type_num) {
+    switch(PyArray_DESCR(a)->type_num) {
     case tBool:
         b = (v != 0);
         NA_SETP(a, Bool, (NA_PTR(a)+offset), b);
@@ -1361,7 +1391,7 @@ void NA_set_Float64(PyArrayObject *a, long offset, Float64 v)
     default:
                      PyErr_Format( PyExc_TypeError,
                              "Unknown type %d in NA_set_Float64",
-                             a->descr->type_num );
+                             PyArray_DESCR(a)->type_num );
                      PyErr_Print();
     }
 }
@@ -1369,127 +1399,127 @@ void NA_set_Float64(PyArrayObject *a, long offset, Float64 v)
 
 Float64 NA_get1_Float64(PyArrayObject *a, long i)
 {
-    long offset = i * a->strides[0];
+    long offset = i * PyArray_STRIDES(a)[0];
     return NA_get_Float64(a, offset);
 }
 
 Float64 NA_get2_Float64(PyArrayObject *a, long i, long j)
 {
-    long offset  = i * a->strides[0]
-        + j * a->strides[1];
+    long offset  = i * PyArray_STRIDES(a)[0]
+        + j * PyArray_STRIDES(a)[1];
     return NA_get_Float64(a, offset);
 }
 
 Float64 NA_get3_Float64(PyArrayObject *a, long i, long j, long k)
 {
-    long offset  = i * a->strides[0]
-        + j * a->strides[1]
-        + k * a->strides[2];
+    long offset  = i * PyArray_STRIDES(a)[0]
+        + j * PyArray_STRIDES(a)[1]
+        + k * PyArray_STRIDES(a)[2];
     return NA_get_Float64(a, offset);
 }
 
 void NA_set1_Float64(PyArrayObject *a, long i, Float64 v)
 {
-    long offset = i * a->strides[0];
+    long offset = i * PyArray_STRIDES(a)[0];
     NA_set_Float64(a, offset, v);
 }
 
 void NA_set2_Float64(PyArrayObject *a, long i, long j, Float64 v)
 {
-    long offset  = i * a->strides[0]
-        + j * a->strides[1];
+    long offset  = i * PyArray_STRIDES(a)[0]
+        + j * PyArray_STRIDES(a)[1];
     NA_set_Float64(a, offset, v);
 }
 
 void NA_set3_Float64(PyArrayObject *a, long i, long j, long k, Float64 v)
 {
-    long offset  = i * a->strides[0]
-        + j * a->strides[1]
-        + k * a->strides[2];
+    long offset  = i * PyArray_STRIDES(a)[0]
+        + j * PyArray_STRIDES(a)[1]
+        + k * PyArray_STRIDES(a)[2];
     NA_set_Float64(a, offset, v);
 }
 
 Complex64 NA_get1_Complex64(PyArrayObject *a, long i)
 {
-    long offset = i * a->strides[0];
+    long offset = i * PyArray_STRIDES(a)[0];
     return NA_get_Complex64(a, offset);
 }
 
 Complex64 NA_get2_Complex64(PyArrayObject *a, long i, long j)
 {
-    long offset  = i * a->strides[0]
-        + j * a->strides[1];
+    long offset  = i * PyArray_STRIDES(a)[0]
+        + j * PyArray_STRIDES(a)[1];
     return NA_get_Complex64(a, offset);
 }
 
 Complex64 NA_get3_Complex64(PyArrayObject *a, long i, long j, long k)
 {
-    long offset  = i * a->strides[0]
-        + j * a->strides[1]
-        + k * a->strides[2];
+    long offset  = i * PyArray_STRIDES(a)[0]
+        + j * PyArray_STRIDES(a)[1]
+        + k * PyArray_STRIDES(a)[2];
     return NA_get_Complex64(a, offset);
 }
 
 void NA_set1_Complex64(PyArrayObject *a, long i, Complex64 v)
 {
-    long offset = i * a->strides[0];
+    long offset = i * PyArray_STRIDES(a)[0];
     NA_set_Complex64(a, offset, v);
 }
 
 void NA_set2_Complex64(PyArrayObject *a, long i, long j, Complex64 v)
 {
-    long offset  = i * a->strides[0]
-        + j * a->strides[1];
+    long offset  = i * PyArray_STRIDES(a)[0]
+        + j * PyArray_STRIDES(a)[1];
     NA_set_Complex64(a, offset, v);
 }
 
 void NA_set3_Complex64(PyArrayObject *a, long i, long j, long k, Complex64 v)
 {
-    long offset  = i * a->strides[0]
-        + j * a->strides[1]
-        + k * a->strides[2];
+    long offset  = i * PyArray_STRIDES(a)[0]
+        + j * PyArray_STRIDES(a)[1]
+        + k * PyArray_STRIDES(a)[2];
     NA_set_Complex64(a, offset, v);
 }
 
 Int64 NA_get1_Int64(PyArrayObject *a, long i)
 {
-    long offset = i * a->strides[0];
+    long offset = i * PyArray_STRIDES(a)[0];
     return NA_get_Int64(a, offset);
 }
 
 Int64 NA_get2_Int64(PyArrayObject *a, long i, long j)
 {
-    long offset  = i * a->strides[0]
-        + j * a->strides[1];
+    long offset  = i * PyArray_STRIDES(a)[0]
+        + j * PyArray_STRIDES(a)[1];
     return NA_get_Int64(a, offset);
 }
 
 Int64 NA_get3_Int64(PyArrayObject *a, long i, long j, long k)
 {
-    long offset  = i * a->strides[0]
-        + j * a->strides[1]
-        + k * a->strides[2];
+    long offset  = i * PyArray_STRIDES(a)[0]
+        + j * PyArray_STRIDES(a)[1]
+        + k * PyArray_STRIDES(a)[2];
     return NA_get_Int64(a, offset);
 }
 
 void NA_set1_Int64(PyArrayObject *a, long i, Int64 v)
 {
-    long offset = i * a->strides[0];
+    long offset = i * PyArray_STRIDES(a)[0];
     NA_set_Int64(a, offset, v);
 }
 
 void NA_set2_Int64(PyArrayObject *a, long i, long j, Int64 v)
 {
-    long offset  = i * a->strides[0]
-        + j * a->strides[1];
+    long offset  = i * PyArray_STRIDES(a)[0]
+        + j * PyArray_STRIDES(a)[1];
     NA_set_Int64(a, offset, v);
 }
 
 void NA_set3_Int64(PyArrayObject *a, long i, long j, long k, Int64 v)
 {
-    long offset  = i * a->strides[0]
-        + j * a->strides[1]
-        + k * a->strides[2];
+    long offset  = i * PyArray_STRIDES(a)[0]
+        + j * PyArray_STRIDES(a)[1]
+        + k * PyArray_STRIDES(a)[2];
     NA_set_Int64(a, offset, v);
 }
 
@@ -1498,7 +1528,7 @@ void NA_set3_Int64(PyArrayObject *a, long i, long j, long k, Int64 v)
 #define NA_SET_CMPLX(a, type, base, cnt, in)                                  \
 {                                                                             \
     int i;                                                                \
-    int stride = a->strides[ a->nd - 1];                                  \
+    int stride = PyArray_STRIDES(a)[ PyArray_NDIM(a) - 1];                                  \
     NA_SET1D(a, type, base, cnt, in);                                     \
     base = NA_PTR(a) + offset + sizeof(type);                             \
     for(i=0; i<cnt; i++) {                                                \
@@ -1512,7 +1542,7 @@ NA_get1D_Float64(PyArrayObject *a, long offset, int cnt, Float64*out)
 {
     char *base = NA_PTR(a) + offset;
 
-    switch(a->descr->type_num) {
+    switch(PyArray_DESCR(a)->type_num) {
     case tBool:
         NA_GET1D(a, Bool, base, cnt, out);
         break;
@@ -1557,7 +1587,7 @@ NA_get1D_Float64(PyArrayObject *a, long offset, int cnt, Float64*out)
     default:
         PyErr_Format( PyExc_TypeError,
                 "Unknown type %d in NA_get1D_Float64",
-                a->descr->type_num);
+                PyArray_DESCR(a)->type_num);
         PyErr_Print();
         return -1;
     }
@@ -1581,7 +1611,7 @@ NA_set1D_Float64(PyArrayObject *a, long offset, int cnt, Float64*in)
 {
     char *base = NA_PTR(a) + offset;
 
-    switch(a->descr->type_num) {
+    switch(PyArray_DESCR(a)->type_num) {
     case tBool:
         NA_SET1D(a, Bool, base, cnt, in);
         break;
@@ -1626,7 +1656,7 @@ NA_set1D_Float64(PyArrayObject *a, long offset, int cnt, Float64*in)
     default:
         PyErr_Format( PyExc_TypeError,
                 "Unknown type %d in NA_set1D_Float64",
-                a->descr->type_num);
+                PyArray_DESCR(a)->type_num);
         PyErr_Print();
         return -1;
     }
@@ -1638,7 +1668,7 @@ NA_get1D_Int64(PyArrayObject *a, long offset, int cnt, Int64*out)
 {
     char *base = NA_PTR(a) + offset;
 
-    switch(a->descr->type_num) {
+    switch(PyArray_DESCR(a)->type_num) {
     case tBool:
         NA_GET1D(a, Bool, base, cnt, out);
         break;
@@ -1681,7 +1711,7 @@ NA_get1D_Int64(PyArrayObject *a, long offset, int cnt, Int64*out)
     default:
         PyErr_Format( PyExc_TypeError,
                 "Unknown type %d in NA_get1D_Int64",
-                a->descr->type_num);
+                PyArray_DESCR(a)->type_num);
         PyErr_Print();
         return -1;
     }
@@ -1705,7 +1735,7 @@ NA_set1D_Int64(PyArrayObject *a, long offset, int cnt, Int64*in)
 {
     char *base = NA_PTR(a) + offset;
 
-    switch(a->descr->type_num) {
+    switch(PyArray_DESCR(a)->type_num) {
     case tBool:
         NA_SET1D(a, Bool, base, cnt, in);
         break;
@@ -1748,7 +1778,7 @@ NA_set1D_Int64(PyArrayObject *a, long offset, int cnt, Int64*in)
     default:
         PyErr_Format( PyExc_TypeError,
                 "Unknown type %d in NA_set1D_Int64",
-                a->descr->type_num);
+                PyArray_DESCR(a)->type_num);
         PyErr_Print();
         return -1;
     }
@@ -1760,14 +1790,14 @@ NA_get1D_Complex64(PyArrayObject *a, long offset, int cnt, Complex64*out)
 {
     char *base = NA_PTR(a) + offset;
 
-    switch(a->descr->type_num) {
+    switch(PyArray_DESCR(a)->type_num) {
     case tComplex64:
         NA_GET1D(a, Complex64, base, cnt, out);
         break;
     default:
         PyErr_Format( PyExc_TypeError,
                 "Unsupported type %d in NA_get1D_Complex64",
-                a->descr->type_num);
+                PyArray_DESCR(a)->type_num);
         PyErr_Print();
         return -1;
     }
@@ -1779,14 +1809,14 @@ NA_set1D_Complex64(PyArrayObject *a, long offset, int cnt, Complex64*in)
 {
     char *base = NA_PTR(a) + offset;
 
-    switch(a->descr->type_num) {
+    switch(PyArray_DESCR(a)->type_num) {
     case tComplex64:
         NA_SET1D(a, Complex64, base, cnt, in);
         break;
     default:
         PyErr_Format( PyExc_TypeError,
                 "Unsupported type %d in NA_set1D_Complex64",
-                a->descr->type_num);
+                PyArray_DESCR(a)->type_num);
         PyErr_Print();
         return -1;
     }
@@ -1808,10 +1838,10 @@ NA_ShapeEqual(PyArrayObject *a, PyArrayObject *b)
                 "NA_ShapeEqual: non-array as parameter.");
         return -1;
     }
-    if (a->nd != b->nd)
+    if (PyArray_NDIM(a) != PyArray_NDIM(b))
         return 0;
-    for(i=0; i<a->nd; i++)
-        if (a->dimensions[i] != b->dimensions[i])
+    for(i=0; i<PyArray_NDIM(a); i++)
+        if (PyArray_DIMS(a)[i] != PyArray_DIMS(b)[i])
             return 0;
     return 1;
 }
@@ -1831,11 +1861,11 @@ NA_ShapeLessThan(PyArrayObject *a, PyArrayObject *b)
                 "NA_ShapeLessThan: non-array as parameter.");
         return -1;
     }
-    mindim = MIN(a->nd, b->nd);
-    aoff = a->nd - mindim;
-    boff = b->nd - mindim;
+    mindim = MIN(PyArray_NDIM(a), PyArray_NDIM(b));
+    aoff = PyArray_NDIM(a) - mindim;
+    boff = PyArray_NDIM(b) - mindim;
     for(i=0; i<mindim; i++)
-        if (a->dimensions[i+aoff] >=  b->dimensions[i+boff])
+        if (PyArray_DIMS(a)[i+aoff] >=  PyArray_DIMS(b)[i+boff])
             return 0;
     return 1;
 }
@@ -2025,15 +2055,14 @@ static int
 getShape(PyObject *a, maybelong *shape, int dims)
 {
     long slen;
-
-    if (PyString_Check(a)) {
+    if (PyBytes_Check(a)) {
         PyErr_Format(PyExc_TypeError,
                 "getShape: numerical sequences can't contain strings.");
         return -1;
     }
 
     if (!PySequence_Check(a) ||
-            (NA_NDArrayCheck(a) && (PyArray(a)->nd == 0)))
+            (NA_NDArrayCheck(a) && (PyArray_NDIM(PyArray(a)) == 0)))
         return dims;
     slen = PySequence_Length(a);
     if (slen < 0) {
@@ -2077,13 +2106,13 @@ setArrayFromSequence(PyArrayObject *a, PyObject *s, int dim, long offset)
     SequenceConstraint mustbe = NOTHING;
     int i, seqlen=-1, slen = PySequence_Length(s);
 
-    if (dim > a->nd) {
+    if (dim > PyArray_NDIM(a)) {
         PyErr_Format(PyExc_ValueError,
                 "setArrayFromSequence: sequence/array dimensions mismatch.");
         return -1;
     }
 
-    if (slen != a->dimensions[dim]) {
+    if (slen != PyArray_DIMS(a)[dim]) {
         PyErr_Format(PyExc_ValueError,
                 "setArrayFromSequence: sequence/array shape mismatch.");
         return -1;
@@ -2096,16 +2125,17 @@ setArrayFromSequence(PyArrayObject *a, PyObject *s, int dim, long offset)
                     "setArrayFromSequence: Can't get a sequence item");
             return -1;
         } else if ((NA_isPythonScalar(o) ||
-                    (NA_NumArrayCheck(o) && PyArray(o)->nd == 0)) &&
+                    (NA_NumArrayCheck(o) && PyArray_NDIM(PyArray(o)) == 0)) &&
                 ((mustbe == NOTHING) || (mustbe == NUMBER))) {
             if (NA_setFromPythonScalar(a, offset, o) < 0)
                 return -2;
             mustbe = NUMBER;
-        } else if (PyString_Check(o)) {
+        } else if (PyBytes_Check(o)) {
             PyErr_SetString( PyExc_ValueError,
                     "setArrayFromSequence: strings can't define numeric numarray.");
             return -3;
         } else if (PySequence_Check(o)) {
+
             if ((mustbe == NOTHING) || (mustbe == SEQUENCE)) {
                 if (mustbe == NOTHING) {
                     mustbe = SEQUENCE;
@@ -2127,7 +2157,7 @@ setArrayFromSequence(PyArrayObject *a, PyObject *s, int dim, long offset)
             return -6;
         }
         Py_DECREF(o);
-        offset += a->strides[dim];
+        offset += PyArray_STRIDES(a)[dim];
     }
     return 0;
 }
@@ -2172,7 +2202,7 @@ _NA_maxType(PyObject *seq, int limit)
         return -1;
     }
     if (NA_NumArrayCheck(seq)) {
-        switch(PyArray(seq)->descr->type_num) {
+        switch(PyArray_DESCR(PyArray(seq))->type_num) {
         case tBool:
             return BOOL_SCALAR;
         case tInt8:
@@ -2196,7 +2226,7 @@ _NA_maxType(PyObject *seq, int limit)
                     "Expecting a python numeric type, got something else.");
             return -1;
         }
-    } else if (PySequence_Check(seq) && !PyString_Check(seq)) {
+    } else if (PySequence_Check(seq) && !PyBytes_Check(seq)) {
         long i, maxtype=BOOL_SCALAR, slen;
 
         slen = PySequence_Length(seq);
@@ -2218,14 +2248,16 @@ _NA_maxType(PyObject *seq, int limit)
         }
         return maxtype;
     } else {
-#if PY_VERSION_HEX >= 0x02030000
         if (PyBool_Check(seq))
             return BOOL_SCALAR;
         else
-#endif
+#if defined(NPY_PY3K)
             if (PyInt_Check(seq))
                 return INT_SCALAR;
             else if (PyLong_Check(seq))
+#else
+            if (PyLong_Check(seq))
+#endif
                 return LONG_SCALAR;
             else if (PyFloat_Check(seq))
                 return FLOAT_SCALAR;
@@ -2255,7 +2287,7 @@ NA_isPythonScalar(PyObject *o)
         PyLong_Check(o) ||
         PyFloat_Check(o) ||
         PyComplex_Check(o) ||
-        (PyString_Check(o) && (PyString_Size(o) == 1));
+        (PyBytes_Check(o) && (PyBytes_Size(o) == 1));
     return rval;
 }
 
@@ -2271,7 +2303,7 @@ NA_isPythonScalar(PyObject *o)
 static PyObject *
 NA_getPythonScalar(PyArrayObject *a, long offset)
 {
-    int type = a->descr->type_num;
+    int type = PyArray_DESCR(a)->type_num;
     PyObject *rval = NULL;
 
     switch(type) {
@@ -2324,9 +2356,9 @@ NA_getPythonScalar(PyArrayObject *a, long offset)
 static int
 NA_overflow(PyArrayObject *a, Float64 v)
 {
-    if ((a->flags & CHECKOVERFLOW) == 0) return 0;
+    if ((PyArray_FLAGS(a) & CHECKOVERFLOW) == 0) return 0;
 
-    switch(a->descr->type_num) {
+    switch(PyArray_DESCR(a)->type_num) {
     case tBool:
         return 0;
     case tInt8:
@@ -2371,7 +2403,7 @@ NA_overflow(PyArrayObject *a, Float64 v)
     default:
         PyErr_Format( PyExc_TypeError,
                 "Unknown type %d in NA_overflow",
-                a->descr->type_num );
+                PyArray_DESCR(a)->type_num );
         PyErr_Print();
         return -1;
     }
@@ -2394,11 +2426,11 @@ _setFromPythonScalarCore(PyArrayObject *a, long offset, PyObject*value, int entr
             return -1;
         NA_set_Int64(a, offset, v);
     } else if (PyLong_Check(value)) {
-        if (a->descr->type_num == tInt64) {
+        if (PyArray_DESCR(a)->type_num == tInt64) {
             v = (Int64) PyLong_AsLongLong( value );
-        } else if (a->descr->type_num == tUInt64) {
+        } else if (PyArray_DESCR(a)->type_num == tUInt64) {
             v = (UInt64) PyLong_AsUnsignedLongLong( value );
-        } else if (a->descr->type_num == tUInt32) {
+        } else if (PyArray_DESCR(a)->type_num == tUInt32) {
             v = PyLong_AsUnsignedLong(value);
         } else {
             v = PyLong_AsLongLong(value);
@@ -2424,7 +2456,7 @@ _setFromPythonScalarCore(PyArrayObject *a, long offset, PyObject*value, int entr
         NA_set_Complex64(a, offset, vc);
     } else if (PyObject_HasAttrString(value, "__tonumtype__")) {
         int rval;
-        PyObject *type = NA_typeNoToTypeObject(a->descr->type_num);
+        PyObject *type = NA_typeNoToTypeObject(PyArray_DESCR(a)->type_num);
         if (!type) return -1;
         value = PyObject_CallMethod(
                 value, "__tonumtype__", "(N)", type);
@@ -2432,14 +2464,14 @@ _setFromPythonScalarCore(PyArrayObject *a, long offset, PyObject*value, int entr
         rval = _setFromPythonScalarCore(a, offset, value, entries+1);
         Py_DECREF(value);
         return rval;
-    } else if (PyString_Check(value)) {
-        long size = PyString_Size(value);
+    } else if (PyBytes_Check(value)) {
+        long size = PyBytes_Size(value);
         if ((size <= 0) || (size > 1)) {
             PyErr_Format( PyExc_ValueError,
                     "NA_setFromPythonScalar: len(string) must be 1.");
             return -1;
         }
-        NA_set_Int64(a, offset, *PyString_AsString(value));
+        NA_set_Int64(a, offset, *PyBytes_AsString(value));
     } else {
         PyErr_Format(PyExc_TypeError,
                 "NA_setFromPythonScalar: bad value type.");
@@ -2451,13 +2483,10 @@ _setFromPythonScalarCore(PyArrayObject *a, long offset, PyObject*value, int entr
 static int
 NA_setFromPythonScalar(PyArrayObject *a, long offset, PyObject *value)
 {
-    if (a->flags & WRITABLE)
-        return _setFromPythonScalarCore(a, offset, value, 0);
-    else {
-        PyErr_Format(
-                PyExc_ValueError, "NA_setFromPythonScalar: assigment to readonly array buffer");
+    if (PyArray_FailUnlessWriteable(a, "array") < 0) {
         return -1;
     }
+    return _setFromPythonScalarCore(a, offset, value, 0);
 }
 
 
@@ -2477,7 +2506,7 @@ NA_ComplexArrayCheck(PyObject *a)
     int rval = NA_NumArrayCheck(a);
     if (rval > 0) {
         PyArrayObject *arr = (PyArrayObject *) a;
-        switch(arr->descr->type_num) {
+        switch(PyArray_DESCR(arr)->type_num) {
         case tComplex64: case tComplex32:
             return 1;
         default:
@@ -2492,8 +2521,8 @@ NA_elements(PyArrayObject  *a)
 {
     int i;
     unsigned long n = 1;
-    for(i = 0; i<a->nd; i++)
-        n *= a->dimensions[i];
+    for(i = 0; i<PyArray_NDIM(a); i++)
+        n *= PyArray_DIMS(a)[i];
     return n;
 }
 
@@ -2694,27 +2723,27 @@ NA_swapAxes(PyArrayObject *array, int x, int y)
 
     if (((PyObject *) array) == Py_None) return 0;
 
-    if (array->nd < 2) return 0;
+    if (PyArray_NDIM(array) < 2) return 0;
 
-    if (x < 0) x += array->nd;
-    if (y < 0) y += array->nd;
+    if (x < 0) x += PyArray_NDIM(array);
+    if (y < 0) y += PyArray_NDIM(array);
 
-    if ((x < 0) || (x >= array->nd) ||
-            (y < 0) || (y >= array->nd)) {
+    if ((x < 0) || (x >= PyArray_NDIM(array)) ||
+            (y < 0) || (y >= PyArray_NDIM(array))) {
         PyErr_Format(PyExc_ValueError,
                 "Specified dimension does not exist");
         return -1;
     }
 
-    temp = array->dimensions[x];
-    array->dimensions[x] = array->dimensions[y];
-    array->dimensions[y] = temp;
+    temp = PyArray_DIMS(array)[x];
+    PyArray_DIMS(array)[x] = PyArray_DIMS(array)[y];
+    PyArray_DIMS(array)[y] = temp;
 
-    temp = array->strides[x];
-    array->strides[x] = array->strides[y];
-    array->strides[y] = temp;
+    temp = PyArray_STRIDES(array)[x];
+    PyArray_STRIDES(array)[x] = PyArray_STRIDES(array)[y];
+    PyArray_STRIDES(array)[y] = temp;
 
-    PyArray_UpdateFlags(array, NPY_UPDATE_ALL);
+    PyArray_UpdateFlags(array, NPY_ARRAY_UPDATE_ALL);
 
     return 0;
 }
@@ -2791,7 +2820,7 @@ NA_NewAllFromBuffer(int ndim, maybelong *shape, NumarrayType type,
 
     if (byteorder != NA_ByteOrder()) {
         PyArray_Descr *temp;
-        temp = PyArray_DescrNewByteorder(dtype, PyArray_SWAP);
+        temp = PyArray_DescrNewByteorder(dtype, NPY_SWAP);
         Py_DECREF(dtype);
         if (temp == NULL) return NULL;
         dtype = temp;
@@ -2819,7 +2848,7 @@ NA_NewAllFromBuffer(int ndim, maybelong *shape, NumarrayType type,
         newdims.len = ndim;
         newdims.ptr = shape;
         newself = (PyArrayObject *)\
-                  PyArray_Newshape(self, &newdims, PyArray_CORDER);
+                  PyArray_Newshape(self, &newdims, NPY_CORDER);
         Py_DECREF(self);
         self = newself;
     }
@@ -2830,20 +2859,20 @@ NA_NewAllFromBuffer(int ndim, maybelong *shape, NumarrayType type,
 static void
 NA_updateAlignment(PyArrayObject *self)
 {
-    PyArray_UpdateFlags(self, NPY_ALIGNED);
+    PyArray_UpdateFlags(self, NPY_ARRAY_ALIGNED);
 }
 
 static void
 NA_updateContiguous(PyArrayObject *self)
 {
-    PyArray_UpdateFlags(self, NPY_CONTIGUOUS | NPY_FORTRAN);
+    PyArray_UpdateFlags(self, NPY_ARRAY_C_CONTIGUOUS | NPY_ARRAY_F_CONTIGUOUS);
 }
 
 
 static void
 NA_updateStatus(PyArrayObject *self)
 {
-    PyArray_UpdateFlags(self, NPY_UPDATE_ALL);
+    PyArray_UpdateFlags(self, NPY_ARRAY_UPDATE_ALL);
 }
 
 static int
@@ -2884,7 +2913,7 @@ NA_getArrayData(PyArrayObject *obj)
         PyErr_Format(PyExc_TypeError,
                 "expected an NDArray");
     }
-    return obj->data;
+    return PyArray_DATA(obj);
 }
 
 /* Byteswap is not a flag of the array --- it is implicit in the data-type */
@@ -3340,20 +3369,55 @@ void *libnumarray_API[] = {
 
 #if (!defined(METHOD_TABLE_EXISTS))
 static PyMethodDef _libnumarrayMethods[] = {
-    {NULL,      NULL, 0, NULL}        /* Sentinel */
+    {NULL, NULL, 0, NULL}        /* Sentinel */
 };
 #endif
 
 /* boiler plate API init */
+#if defined(NPY_PY3K)
+
+#define RETVAL m
+
+static struct PyModuleDef moduledef = {
+        PyModuleDef_HEAD_INIT,
+        "_capi",
+        NULL,
+        -1,
+        _libnumarrayMethods,
+        NULL,
+        NULL,
+        NULL,
+        NULL
+};
+
+PyMODINIT_FUNC PyInit__capi(void)
+#else
+
+#define RETVAL
+
 PyMODINIT_FUNC init_capi(void)
+#endif
 {
-    PyObject *m = Py_InitModule("_capi", _libnumarrayMethods);
+    PyObject *m;
     PyObject *c_api_object;
 
     _Error = PyErr_NewException("numpy.numarray._capi.error", NULL, NULL);
 
     /* Create a CObject containing the API pointer array's address */
+#if defined(NPY_PY3K)
+    m = PyModule_Create(&moduledef);
+#else
+    m = Py_InitModule("_capi", _libnumarrayMethods);
+#endif
+
+#if defined(NPY_PY3K)
+    c_api_object = PyCapsule_New((void *)libnumarray_API, NULL, NULL);
+    if (c_api_object == NULL) {
+        PyErr_Clear();
+    }
+#else
     c_api_object = PyCObject_FromVoidPtr((void *)libnumarray_API, NULL);
+#endif
 
     if (c_api_object != NULL) {
         /* Create a name for this object in the module's namespace */
@@ -3362,13 +3426,16 @@ PyMODINIT_FUNC init_capi(void)
         PyDict_SetItemString(d, "_C_API", c_api_object);
         PyDict_SetItemString(d, "error", _Error);
         Py_DECREF(c_api_object);
-    } else {
-        return;
     }
-    if (PyModule_AddObject(m, "__version__",
-                PyString_FromString("0.9")) < 0) return;
-
-    if (_import_array() < 0) return;
+    else {
+        return RETVAL;
+    }
+    if (PyModule_AddObject(m, "__version__", PyUString_FromString("0.9")) < 0) {
+        return RETVAL;
+    }
+    if (_import_array() < 0) {
+        return RETVAL;
+    }
     deferred_libnumarray_init();
-    return;
+    return RETVAL;
 }
