@@ -1,16 +1,21 @@
-# Lifted from Precision.py.  This is for compatibility only.
-#
-#  The character strings are still for "new" NumPy
-#   which is the only Incompatibility with Numeric
+"""
+
+Lifted from Precision.py.  This is for compatibility only.
+
+The character strings are still for "new" NumPy
+which is the only Incompatibility with Numeric
+
+"""
+from __future__ import division, absolute_import, print_function
 
 __all__ = ['Character', 'Complex', 'Float',
            'PrecisionError', 'PyObject', 'Int', 'UInt',
-           'UnsignedInteger', 'string', 'typecodes', 'zeros']
+           'UnsignedInt', 'UnsignedInteger', 'string', 'typecodes', 'zeros']
 
-from functions import zeros
+from .functions import zeros
 import string   # for backwards compatibility
 
-typecodes = {'Character':'c', 'Integer':'bhil', 'UnsignedInteger':'BHI', 'Float':'fd', 'Complex':'FD'}
+typecodes = {'Character':'c', 'Integer':'bhil', 'UnsignedInteger':'BHIL', 'Float':'fd', 'Complex':'FD'}
 
 def _get_precisions(typecodes):
     lst = []
@@ -33,7 +38,8 @@ def _lookup(table, key, required_bits):
     for bits, typecode in lst:
         if bits >= required_bits:
             return typecode
-    raise PrecisionError, key+" of "+str(required_bits)+" bits not available on this system"
+    raise PrecisionError(key + " of " + str(required_bits) +
+            " bits not available on this system")
 
 Character = 'c'
 
@@ -67,8 +73,7 @@ try:
     __all__.extend(['UnsignedInt128', 'UInt128'])
 except(PrecisionError):
     pass
-UnsignedInteger = 'u'
-UInt = UnsignedInteger
+UInt = UnsignedInt = UnsignedInteger = 'u'
 
 try:
     Int0 = _lookup(_code_table, 'Integer', 0)
