@@ -1,8 +1,10 @@
 #!/usr/bin/env python
+from __future__ import division, print_function
+
 
 def configuration(parent_package='',top_path=None):
     from numpy.distutils.misc_util import Configuration
-    config = Configuration('numpy',parent_package,top_path)
+    config = Configuration('numpy', parent_package, top_path)
     config.add_subpackage('distutils')
     config.add_subpackage('testing')
     config.add_subpackage('f2py')
@@ -24,4 +26,4 @@ def configuration(parent_package='',top_path=None):
     return config
 
 if __name__ == '__main__':
-    print 'This is the wrong setup.py file to run'
+    print('This is the wrong setup.py file to run')
