@@ -1,3 +1,5 @@
+from __future__ import division, absolute_import, print_function
+
 import sys
 
 import numpy as np
@@ -18,10 +20,10 @@ class TestLoadLibrary(TestCase):
         try:
             cdll = load_library('multiarray',
                                 np.core.multiarray.__file__)
-        except ImportError, e:
+        except ImportError as e:
             msg = "ctypes is not available on this python: skipping the test" \
                   " (import error was: %s)" % str(e)
-            print msg
+            print(msg)
 
     @dec.skipif(not _HAS_CTYPE, "ctypes not available on this python installation")
     @dec.knownfailureif(sys.platform=='cygwin', "This test is known to fail on cygwin")
@@ -34,20 +36,20 @@ class TestLoadLibrary(TestCase):
                 cdll = load_library('multiarray%s' % so,
                                     np.core.multiarray.__file__)
             except ImportError:
-                print "No distutils available, skipping test."
-        except ImportError, e:
+                print("No distutils available, skipping test.")
+        except ImportError as e:
             msg = "ctypes is not available on this python: skipping the test" \
                   " (import error was: %s)" % str(e)
-            print msg
+            print(msg)
 
 class TestNdpointer(TestCase):
     def test_dtype(self):
         dt = np.intc
         p = ndpointer(dtype=dt)
-        self.assert_(p.from_param(np.array([1], dt)))
+        self.assertTrue(p.from_param(np.array([1], dt)))
         dt = '<i4'
         p = ndpointer(dtype=dt)
-        self.assert_(p.from_param(np.array([1], dt)))
+        self.assertTrue(p.from_param(np.array([1], dt)))
         dt = np.dtype('>i4')
         p = ndpointer(dtype=dt)
         p.from_param(np.array([1], dt))
@@ -58,42 +60,42 @@ class TestNdpointer(TestCase):
         dtdescr = {'names' : dtnames, 'formats' : dtformats}
         dt = np.dtype(dtdescr)
         p = ndpointer(dtype=dt)
-        self.assert_(p.from_param(np.zeros((10,), dt)))
+        self.assertTrue(p.from_param(np.zeros((10,), dt)))
         samedt = np.dtype(dtdescr)
         p = ndpointer(dtype=samedt)
-        self.assert_(p.from_param(np.zeros((10,), dt)))
+        self.assertTrue(p.from_param(np.zeros((10,), dt)))
         dt2 = np.dtype(dtdescr, align=True)
         if dt.itemsize != dt2.itemsize:
             self.assertRaises(TypeError, p.from_param, np.zeros((10,), dt2))
         else:
-            self.assert_(p.from_param(np.zeros((10,), dt2)))
+            self.assertTrue(p.from_param(np.zeros((10,), dt2)))
 
     def test_ndim(self):
         p = ndpointer(ndim=0)
-        self.assert_(p.from_param(np.array(1)))
+        self.assertTrue(p.from_param(np.array(1)))
         self.assertRaises(TypeError, p.from_param, np.array([1]))
         p = ndpointer(ndim=1)
         self.assertRaises(TypeError, p.from_param, np.array(1))
-        self.assert_(p.from_param(np.array([1])))
+        self.assertTrue(p.from_param(np.array([1])))
         p = ndpointer(ndim=2)
-        self.assert_(p.from_param(np.array([[1]])))
+        self.assertTrue(p.from_param(np.array([[1]])))
 
     def test_shape(self):
-        p = ndpointer(shape=(1,2))
-        self.assert_(p.from_param(np.array([[1,2]])))
-        self.assertRaises(TypeError, p.from_param, np.array([[1],[2]]))
+        p = ndpointer(shape=(1, 2))
+        self.assertTrue(p.from_param(np.array([[1, 2]])))
+        self.assertRaises(TypeError, p.from_param, np.array([[1], [2]]))
         p = ndpointer(shape=())
-        self.assert_(p.from_param(np.array(1)))
+        self.assertTrue(p.from_param(np.array(1)))
 
     def test_flags(self):
-        x = np.array([[1,2,3]], order='F')
+        x = np.array([[1, 2], [3, 4]], order='F')
         p = ndpointer(flags='FORTRAN')
-        self.assert_(p.from_param(x))
+        self.assertTrue(p.from_param(x))
         p = ndpointer(flags='CONTIGUOUS')
         self.assertRaises(TypeError, p.from_param, x)
         p = ndpointer(flags=x.flags.num)
-        self.assert_(p.from_param(x))
-        self.assertRaises(TypeError, p.from_param, np.array([[1,2,3]]))
+        self.assertTrue(p.from_param(x))
+        self.assertRaises(TypeError, p.from_param, np.array([[1, 2], [3, 4]]))
 
 
 if __name__ == "__main__":
